#!/usr/bin/perl -w

#   Copyright (C) 2006 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

package eqn_string;

use warnings 'all';

use Exporter 'import';
@EXPORT_OK = qw(create_evaluable_eqn create_evaluable_eqn_immed replaceBackTickedLocals min max misc str_to_varlist);

use strict 'refs';

use misc2 qw(inAry inDict func_fail);
use Scalar::Util qw(looks_like_number);

$builtins =  [ "exp","sqrt", "log","sin","tan","cos","int","abs","step","min","max","looks_like_number", "eq", "ne", "xtile", "rand", "randnorm" ];

$builtins_requiring_varnames = [ "xtile" ];


sub create_evaluable_eqn {
 my $dm    = shift;
 my $gb    = shift;
 my $eqn   = shift;
 my $doif  = shift;

 if (!defined($dm) || !defined($gb)) {
  return { status => "fail; (internal) -- passed undefined structures" };
 }

 if (!defined($doif)) { $doif = 0; };

 my $r = convert_eqn($dm,$gb,$eqn,0);

 if (func_fail($r)) {
  return $r;
 } else {
  if ($doif) {
   ($r->{ifary}, 
    $r->{forward_iftable},
    $r->{reverse_iftable}) = create_if_array($dm,$gb,$r->{expression});
  }
 }

 return $r;

}

sub create_evaluable_eqn_immed {
 my $dm    = shift;
 my $gb    = shift;
 my $eqn   = shift;
# print "IN EQN is $eqn\n";
 my $r = convert_eqn($dm,$gb,$eqn,1);
# print "OUT EQN is $r->{status} $r->{expression}\n";
 return $r;
}




sub convert_eqn {
 my $dm    = shift;
 my $gb    = shift;
 my $eqn   = shift;
 my $immed = shift;

# print "NEW EQN is $eqn\n";

 # bypass all parsing for "direct perl" mode 
 # This exists mostly for my own debugging purposes, though 
 # This can be used to run expressions that would be impossible
 # or awkward with the simplfied syntax supported
 if ($eqn =~ /^perl/) {
  $eqn =~ s/^perl\s*//;
  return { status => "ok",
	   expression => $eqn
         }
 }

 # add space to cause last element to flush and change 
 # semicolons back to commas.
 $eqn .= " ";
 $eqn =~ s/;/,/g;

 my $state     = "s0";
 my $word      = "";
 my $indx      = "";
 my $elemcount = 0;
 my $elems = undef;
 my $lastch    = "";
 for (my $i=0;$i<length($eqn);$i++) {
  my $inch = substr($eqn,$i,1);
#  print "eqn " . $inch . " $word " . $state  . " " . $elemcount . "\n";
  if ($state eq "s0") {
   if (($inch eq "e") && ($lastch =~ /\d/)) {
    $elems{$elemcount}{type}   = "other";
    $elems{$elemcount++}{val}  = $inch;
   } elsif ($inch =~ /[_a-z]/) {
    $state = "s1";
    $word  .= $inch;
#   } elsif ($inch eq "\"") {
   } elsif ($inch eq "\'") {
    $state = "s2";
   } else {
    $elems{$elemcount}{type}   = "other";
    $elems{$elemcount++}{val}  = $inch;
   }
  } elsif ($state eq "s2") {
#   if ($inch eq "\"") {
   if ($inch eq "\'") {
    $elems{$elemcount}{type}  = "string";
    $elems{$elemcount++}{val} = $word;
    $state = "s0";
    $word = "";
   } else {
    $word .= $inch;
   }
  } elsif ($state eq "s1") {
   if (($inch eq "(") && ($word eq "r")) {
    $state = "s3r";
    $word  .= $inch;
   } elsif (($inch eq "(") && ($word eq "g")) {
    $state = "s3g";
    $word  .= $inch;
   } elsif (($inch eq "(") && (inAry($word,$builtins_requiring_varnames))) {
    $state = "s4";
    $word  .= $inch;
   } elsif ($inch =~ /\w/) {
    $word .= $inch;
   } elsif ($inch eq "[") {
    $state = "s5";
   } else {
    $elems{$elemcount}{type}  = "word";
    delete $elems{$elemcount}{indx};
    $elems{$elemcount++}{val} = $word;
    $word = "";
#    if ($inch eq "\"") {
    if ($inch eq "\'") {
     $state = "s2";
    } else {
     $state = "s0";
     $elems{$elemcount}{type} = "other";
     $elems{$elemcount++}{val} = $inch;
    }
   }
  } elsif ($state eq "s3r") {
   if ($inch eq ")") {
    $word .= $inch;
    $state = "s0";
    $elems{$elemcount}{type} = "rval";
    $elems{$elemcount++}{val} = $word;
   } else {
    $word .= $inch;
   }
  } elsif ($state eq "s3g") {
   if ($inch eq ")") {
    $word .= $inch;
    $state = "s0";
    $elems{$elemcount}{type} = "gval";
    $elems{$elemcount++}{val} = $word;
   } else {
    $word .= $inch;
   }
  } elsif ($state eq "s4") {
   if ($inch eq ")") {
    $word .= $inch;
    $state = "s0";
    $elems{$elemcount}{type} = "xtile";
    $elems{$elemcount++}{val} = $word;
    $state = "s0";
   } else {
    $word .= $inch;
   }
  } elsif ($state eq "s5") {
   if ($inch eq "]") {
    $state = "s0";
    $elems{$elemcount}{type} = "word";
    $elems{$elemcount}{val}  = $word;
    $elems{$elemcount}{indx} = $indx;
    $elemcount++;
    $indx = "";
    $word = "";
   } else {
    $indx .= $inch;
   }
  } else {
   die "Equation string: we should never be here.\n";
  }
  $lastch = $inch;
 }

# print "FINAL STATE $state WORD $word\n";
 if (($state eq "s4") || 
     ($state eq "s3r") ||
     ($state eq "s3g") ||
     ($state eq "s5")) {
  return { status => "fail; parse error in equation" }
 }

 for (my $i=0;$i<$elemcount;$i++) {
#  print "ELEM $i TYPE $elems{$i}{type} VAL \"$elems{$i}{val}\"\n";
#  if (defined($elems{$i}{indx})) { print " index:" .  $elems{$i}{indx} . "\n"; }
  if ($elems{$i}{type} eq "word") {
   my $cv = $elems{$i}{val};
   if (inAry($cv,$builtins)) {
    $elems{$i}{type} = "func";
   } elsif ($cv eq "_n") {
    $elems{$i}{type} = "_n";
   } elsif ($cv eq "_ln") {
    $elems{$i}{type} = "_ln";
   } elsif (!$immed && inDict($cv,$dm->{data})) {
    $elems{$i}{type} = "var";
   } elsif (inDict($cv,$dm->{locals})) {
    $elems{$i}{type} = "local";
   } else {
    return { status => "fail; unknown element \"$cv\" in expression. Variable not found?" }
   }
  }
 }

 $r->{status} = "ok";

 if (0) {
  for (my $i=0;$i<$elemcount;$i++) {
   print "-- element: " . $elems{$i}{type} . " " . $elems{$i}{val};
   if (defined($elems{$i}{indx})) { print " idx: $elems{$i}{indx}"; }
   print "\n";
  }
 }

 for (my $i=0;$i<$elemcount;$i++) {
  my $et  = $elems{$i}{type};
  my $old = $elems{$i}{val};
  if      ($et eq "word") {
   $elems{$i}{val} = " " . $old;
  } elsif ($et eq "rval") {
   $old =~ s/^r\(//;
   $old =~ s/\)$//;
   $old =~ s/;/,/;
   my @rps = split(/,/,$old);
   $r = rvalAryToDeref($gb,\@rps);
   if (func_fail($r)) {
    return $r;
   } else {
    $new = $r->{rval_str};
    $elems{$i}{val} = $new;
   }
  } elsif ($et eq "gval") {
   $old =~ s/^g\(//;
   $old =~ s/\)$//;
   $old =~ s/;/,/;
   my @rps = split(/,/,$old);
   $r = gvalAryToDeref($gb,\@rps);
   if (func_fail($r)) {
    return $r;
   } else {
    $new = $r->{gval_str};
    $elems{$i}{val} = $new;
   }
  } elsif ($et eq "func") {
   # I think functions are ok the way they are.
  } elsif ($et eq "_n") {
   my $new  = "(\$row)";
   $elems{$i}{val} = $new;
  } elsif ($et eq "_ln") {
   my $new = "(\$lrow)";
   $elems{$i}{val} = $new;
  } elsif ($et eq "xtile") {
   $old =~ s/\)\s*$//;
   (my $fn, my $argstr) = split(/\(/,$old);
   my @args = split(/,/,$argstr);
   my $newargs = "\$self,\$row,";
   foreach my $a (@args) {
    if (!looks_like_number($a)) {
     if (inDict($a,$dm->{data})) {
      $a = "\"" . $a . "\"";
     } else {
      return { status => "fail; Variable $a not in table" };
     }
    }
    $newargs .= $a . ",";
   }
   chop $newargs;
   my $new = '(' . $fn . '(' . $newargs . ')' . ')';
   $elems{$i}{val} = $new;
  } elsif ($et eq "string") {
   my $new = "\'" . $old . "\'";
   $elems{$i}{val} = $new;
  } elsif ($et eq "var") {
   if (defined($elems{$i}{indx})) {
    my $indx = $elems{$i}{indx};
    # !!!! _ln and _n cannot coexist in the same index !!!

    if ($indx =~ /_ln/) {
     $indx =~ s/_ln/\$lrow/g;
     $indx = "(\$lrows->[$indx])";
    } else {
     $indx =~ s/_n/\$row/g;
    }

    my $maxary = '(@{$self->{dm}{data}{' . $old . '}})';
    my $new = "";

    $new .= "(((($indx) >= 0) && (($indx) < $maxary)) ? ";
    $new .= '($self->{dm}{data}{' . $old . '}[' . $indx. '])';
    $new .= ' : undef)';

    $elems{$i}{val} = $new;
   } else {
    my $new  = '($self->{dm}{data}{' . $old . '}[$row])';
    $elems{$i}{val} = $new;
   }
  } elsif ($et eq "local") {
   my $new  = '($self->{dm}{locals}{' . $old. '})';
   $elems{$i}{val} = $new;
  } elsif ($et eq "other") {
   # others are just fine the way they are
  } else {
   return { status => "fail; expression unknown parse element $et" }
  }
 }


 my $eval_str = "";
 for (my $i=0;$i<$elemcount;$i++) {
  $eval_str .= $elems{$i}{val};
  $r->{expression} = $eval_str;
  $r->{status}     = "ok";
 }
 # print "EVAL STR $eval_str\n";
 return $r;
 
}

sub rvalAryToDeref {
 my $gb = shift;
 my $ar = shift;
 my @a  = @{$ar};
 my $s  = "";
 my $r = {};
 $r->{status}   = "ok";
 foreach $e (@a) {
  $s .= '{' . $e . '}';
 }
 $s = '($self->{gb}{rval}' . $s . ')';

 # most evaluation of these strings take place in the context
 # of an object with self/gb/dm members. This one doesn't so we
 # fake it from references in parameter list
 my $self = {};
 $self->{gb} = $gb;

 if (defined(eval($s))) {
  $r->{rval_str} = $s;
 } else {
  $r->{status} = "fail; rval requested $s not defined";
 }
 return $r;
};

sub gvalAryToDeref {
 my $gb = shift;
 my $ar = shift;
 my @a  = @{$ar};
 my $s  = "";
 my $r = {};
 $r->{status}   = "ok";
 foreach $e (@a) {
  $s .= '{' . $e . '}';
 }
 $s = '($self->{gb}' . $s . ')';

 # most evaluation of these strings take place in the context
 # of an object with self/gb/dm members. This one doesn't so we
 # fake it from references in parameter list
 my $self = {};
 $self->{gb} = $gb;

 if (defined(eval($s))) {
  $r->{gval_str} = $s;
 } else {
  $r->{status} = "fail; gval requested $s not defined";
 }
 return $r;
};

sub create_if_array {
 my $dm  = shift;
 my $gb  = shift;
 my $eqn = shift;

 my $self = {};
 $self->{dm} = $dm;
 $self->{gb} = $gb;

 my @ifary = ();
 my $rev_iftable = {};
 my $fwd_iftable = [];
 my @vars = keys %{$dm->{data}};
 my $somevar = shift @vars;
 my @numrows_ary = @{$dm->{data}{$somevar}};
 my $numrows = @numrows_ary;
 if ($somevar) {
  my $ct=0;
  for (my $row=0;$row<$numrows;$row++) {
   $ifary[$row] = eval($eqn);
   if ($@) {
    $ifary[$row] = 0;
   }
   if ($ifary[$row]) {
    $rev_iftable->{$row}  = $ct;
    $fwd_iftable->[$ct++] = $row;
   }
  }
 }
 return (\@ifary, $fwd_iftable, $rev_iftable);
}

sub replaceBackTickedLocals {
 my $dm = shift;
 my $gb = shift;
 my $is = shift;

 my $self = {};
 $self->{dm} = $dm;

# print "IS is $is\n";
 my $os = "";
 my $in_back;
 my $back_str = "";
 for (my $i=0;$i<length($is);$i++) {
  my $inch = substr($is,$i,1);
  if ($in_back) {
   if ($inch eq "`") {
    $in_back = 0;
    my $r = create_evaluable_eqn_immed($dm,$gb,$back_str);
    if (!func_fail($r)) {
     my $val = eval($r->{expression});
     if ($@) {
      $val = '';
     } elsif (!defined($val)) {
      $val = '';
     } elsif (looks_like_number($val)) {
      if ($val == 0) {
       $val = "0";
      }
     }
     $os .= $val;
    } else {
     return $r;
    }
   } else {
    $back_str .= $inch;
   }
  } else {
   if ($inch eq "`") {
    $back_str = "";
    $in_back = 1;
   } else {
    $os .= $inch;
   }
  }
 }
 if ($in_back) {
  my $r = create_evaluable_eqn_immed($dm,$gb,$back_str);
  if (!func_fail($r)) {
   my $val = eval($r->{expression});
   if ($@) {
    $val = '';
   } elsif (!defined($val)) {
    $val = '';
   } elsif (looks_like_number($val)) {
    if ($val == 0) {
     $val = "0";
    }
   }
   $os .= $val;
  } else {
   return $r;
  }
 }
 return { status => "ok",
	  cmdline => $os };
}


sub str_to_varlist {
 my $is = shift;
 my $dm = shift;
 my $possible_vars;
 my @pieces = split(/\W+/,$is);
 foreach $piece (@pieces) {
  if (!inAry($piece,$builtins) &&
      !inAry($piece,$builtins_requiring_varnames) &&
      !inDict($piece,$dm->{locals}) &&
      inDict($piece,$dm->{data})) {
   $possible_vars->{$piece} = 1;
  }
 }
 return keys (%{$possible_vars});
}

1;

