#!/usr/bin/perl -w

#   Copyright (C) 2006 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

use fgroup;
package fgroup::help;
BEGIN { @ISA = qw(fgroup); };

use Exporter 'import';
@EXPORT_OK = qw();

use strict;
use FindBin qw($Bin);

# override default constructor in fgroup to load help
# file
sub new {
 my $proto           = shift;
 my $oo              = shift;
 my $gb              = shift;
 my $dm              = shift;
 my $self            = {};

 $self->{dm}   = $dm;
 $self->{gb}   = $gb;
 $self->{oo}   = $oo;
 $self->{help} = undef;
 my $class           = ref($proto) || $proto;
 bless ($self,$class);

 my $hf = $Bin . "/" . $gb->{helpfile};
 if (-e $hf) {
   $self->{help} = load_help($oo,$hf);
 }
 return $self;
}

sub commands {
 my $self = shift;
 my @cmds = ("help", "search");
 return @cmds;
}

sub search {
 my $self = shift;
 my $a    = shift;
 my $oo   = $self->{oo};
 my $r    = {};
 $r->{status} = "ok";
 if (!defined($self->{help})) {
  $oo->tdisp('warning',"Sorry, no help available.\n");
  return $r;
 }
 my $matching_topics = {};
 foreach my $arg (@{$a->{args}}) {
  foreach my $topic (keys %{$self->{help}}) {
   for (my $i=0;$i<@{$self->{help}{$topic}};$i++) {
    if ($self->{help}{$topic}[$i] =~ /$arg/) {
     $matching_topics->{$topic} = $i;
    }
   }
  }
 }
 if (scalar keys %{$matching_topics}) {
  $oo->disp(" See: \n");
  foreach my $topic (keys %{$matching_topics}) {
   $oo->disp("   ");
   $oo->tdisp('help_help',"$topic");
   $oo->disp("\n");
  }
  $oo->disp("\n");
 } else {
  $oo->disp("No matches found.\n\n");
 }
 return $r;
}

sub help {
 my $self = shift;
 my $a    = shift;
 my $oo   = $self->{oo};

 my $r = {};
 $r->{status} = "ok";

 if (!defined($self->{help})) {
  $oo->tdisp('warning',"Sorry, no help available.\n");
  return $r;
 };

 my $request = "generic";
 if (defined($a->{args}[0])) {
  $request = $a->{args}[0];
 }

 if (defined($self->{help}{$request})) {
  my $os = "\n";
  $os .= "_____ " . $request . " ";
  for (my $i=0;$i<(60-length($request));$i++) { $os .= "_"; };
  $os .= "\n";
  $oo->disp($os);
  if ($request eq "topics") {
   $oo->disp("\nAvailable help topics:\n");
   my $count = 0;
   foreach my $t (sort keys %{$self->{help}}) {
    $oo->tdisp('help_help',$t);
    $oo->disp(", ");
    if (($count++ % 7) == 6) {
     $oo->disp("\n");
    };
   }
   $oo->disp("\n");
  } else {
   foreach my $l (@{$self->{help}{$request}}) {
    if ($l =~ /^\s*\.[\s\w]*/) {
     $oo->tdisp('help_cmd',$l);
     $oo->disp("\n");
    } elsif ($l =~ /help:\w*/) {
     my @pieces = split(/\s/,$l);
     foreach my $piece (@pieces) {
      if ($piece =~ s/(.*)help:(\w*)(.*)//) {
       $oo->disp($1);
       my $topic = $2;
       $oo->tdisp('help_help',$topic);
       $oo->disp($piece . "$3 ");
      } else {
       $oo->disp($piece . " ");
      }
     }
     $oo->disp("\n");
    } else {
     $oo->disp($l . "\n");
    }
   }
  }
  $os = "";
  for (my $i=0;$i<(60-length($request));$i++) { $os .= "_"; };
  $os .= " " . $request . " _____\n";
  $oo->disp($os . "\n");
 } else {
  $oo->tdisp('warning',"Sorry no help found for $request. Have you tried \'help topics\'?\n");
 }

 return $r;
}


sub load_help {
 my $oo   = shift; 
 my $hfn  = shift;
 my $h = {};
 my $ok = open(IF,"<" . $hfn);
 if (!$ok) {
  $oo->tdisp('warning',"Could not find help file $hfn\n");
  return;
 } 

 my $currname    = "generic";
 my $currary     = [];
 my $count       = 0;
 while (<IF>) {
  my $il = $_;
  chomp $il;
  if ($il =~ /^=(\w+)$/) {
   $h->{$currname} = $currary;
   $currname = $1;
   $currary = [];
   $count   = 0;
  } else {
   if ($il =~ /^#/) {
   } else {
    $currary->[$count++] = $il;
   }
  }
 }
 if ($count) {
  $h->{$currname} = $currary;
 } 
 $h->{topics}[0] = "";
 return $h;
}
1;
