#!/usr/bin/perl -w

#   Copyright (C) 2006 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

use fgroup;
package fgroup::insheet;

BEGIN { @ISA = qw(fgroup); 
 if ($^O eq "MSWin32") {
  require "Win32/OLE.pm";
  import Win32::OLE;
  require "Win32/OLE/Const.pm";
  import Win32::OLE::Const;
 }
};

use Exporter 'import';

use strict;
use csv_funcs qw(read_csv read_csv_old);
use misc2 qw(tstFlg inAry func_fail);
use Scalar::Util qw(looks_like_number);
use gui qw(tk_updateVars);
use filewrap qw(isUrl);



sub commands {
 my $self = shift;
 $self->{ExcelOLE} = undef;
 $self->{ExcelOLEConstants} = undef;
 my @cmds = ("insheet", "insheet_old", "outsheet", "mergesheet");
 return @cmds;
}


sub cleanup {
 my $dm       = shift;
 my $rowcount = shift;
 my @vars = keys %{$dm->{data}};
 foreach my $var (@vars) {
  my $numeric = "numeric";
  for (my $row=0;$row<$rowcount;$row++) {
   if (!looks_like_number($dm->{data}{$var}[$row])) {
    $numeric = "categorical";
   }
  }
  $dm->{info}{$var}{type} = $numeric;
 }
}


sub mergesheet {
 my $self   = shift;
 my $a      = shift;
 my $oo     = $self->{oo};

 my $r      = {};
 $r->{status} = "ok";
 my $fname = "";
 if ($a->{have_using}) {
  $fname = $a->{using};
 } else {
  return { status => "fail; missing \"using\" value" }
 }

 if (!defined($a->{varary}[0])) {
  return { status => "fail; no merge variable specified" }
 }

 foreach my $v (@{$a->{varary}}) {
  if ($v->{state} ne "exist") {
   return { status => "fail; specified variable $v->{name} does not existin current database"};
  }
 }

 # for now, only support one merge var, but in the future, maybe fix
 # this to handle as many as the user wishes to specify. A match would 
 # be a match of all of them. A user workaround for cases where multiply
 # merge vars would be desirable is to create a new var that hashes together
 # all the desired merge vars before merging. 
 my $mergevar = $a->{varary}[0]{name};

 my ($names, $aa);
 ($r, $names, $aa) =  $self->insheet_helper($oo,$a,$fname);
 if (func_fail($r)) {
  return $r;
 } 

 for (my $i=0;$i<@{$names};$i++) { $names->[$i] = lc($names->[$i]); };

 foreach my $v (@{$a->{varary}}) {
  if (!inAry($v->{name},$names)) {
   return { status => "fail; merge variable $v->{name} not found in merge file" };
  }
 }

 my $mergeID = 0;
 my $ID_found = 0;
 while (!$ID_found) {
  if ($mergevar eq $names->[$mergeID]) {
   $ID_found = 1;
  } else {
   $mergeID++;
  }
 }

 my $masrows = @{$self->{dm}{data}{lc($names->[0])}};
 my $newrows = @{$aa->[0]};
 $r->{mergestats}{none} = 0;
 $r->{mergestats}{one}  = 0;
 $r->{mergestats}{many} = 0;

 for (my $i=0;$i<$newrows;$i++) {
  my $matchcount = 0;
  for (my $j=0;$j<$masrows;$j++) {
   if ((looks_like_number($aa->[$mergeID][$i]) &&
        looks_like_number($self->{dm}{data}{$mergevar}[$j]) &&
        ($aa->[$mergeID][$i] == $self->{dm}{data}{$mergevar}[$j])) ||
       (($aa->[$mergeID][$i] eq $self->{dm}{data}{$mergevar}[$j]))
      ) {
    $matchcount++;
    my $idx = 0;
    foreach my $nv (@{$names}) {
     $self->{dm}{data}{$nv}[$j] = $aa->[$idx][$i];
     $idx++;
    }
    $self->{dm}{data}{_merge}[$j] = $matchcount;
   }
  }
  if ($matchcount == 0) { $r->{mergestats}{none}++; };
  if ($matchcount == 1) { $r->{mergestats}{one}++; };
  if ($matchcount >  1) { $r->{mergestats}{many}++; };
 }

 for (my $j=0;$j<$masrows;$j++) {
  if (!defined($self->{dm}{data}{_merge}[$j])) {
   $self->{dm}{data}{_merge}[$j] = 0;
   foreach my $nv (@{$names}) {
    $self->{dm}{data}{lc($nv)}[$j] = "__undef";
   }
  }
 }

 cleanup($self->{dm},$masrows);

 $oo->disp(sprintf("(merge file: %d vars, %d obs)\n",scalar @{$names},$newrows));
 my $report = "\nMerge statistics:\n  " . 
               $r->{mergestats}{none} .  " non-matches,\n  " .
	       $r->{mergestats}{one}  .  " unique matches,\n  " . 
	       $r->{mergestats}{many} .  " non-unique matches.\n\n";
 $oo->disp($report);

 if ($self->{gb}{using_gui}) {
  tk_updateVars($self->{dm});
 }
 return $r;
}

sub insheet {
 my $self   = shift;
 my $a      = shift;
 my $oo     = $self->{oo};

 if ($self->{gb}{dirty}) {
  return { status => "fail; No. Data would be lost. Clear or save first." }
 }
 my $fname  = "";
 if ($a->{have_using}) {
  $fname = $a->{using}
 } else {
  return { status => "fail; missing \"using\" value" }
 }

 my $oo = $self->{oo};

 my ($r, $names, $aa) =  $self->insheet_helper($oo,$a,$fname);
 if (func_fail($r)) {
  return $r;
 } 

 my $col = 0;
 foreach my $name (@{$names}) {
  $self->{dm}{data}{lc($name)} = $aa->[$col];
  $col++;
 }
 $self->{gb}{dirty} = 1;
 $r->{status} = "ok";
 my $numrows = @{$self->{dm}{data}{lc($names->[0])}};

 cleanup($self->{dm},$numrows);

 $oo->disp(sprintf("(%d vars, %d obs)\n\n",$col,$numrows));

 if ($self->{gb}{using_gui}) {
  tk_updateVars($self->{dm});
 } 
 return $r;
}


sub insheet_old {
 my $self   = shift;
 my $a      = shift;

 if ($self->{gb}{dirty}) {
  return { status => "fail; No. Data would be lost. Clear or save first." }
 }
 my $fname  = "";
 if ($a->{have_using}) {
  $fname = $a->{using}
 } else {
  return { status => "fail; missing using val" }
 }

 my $r = {};
 my $oo = $self->{oo};

 my $tdf = tstFlg($a,"tab");

 if (!(-e $fname)) {
  $r->{status} = "error -- could not find file $fname"; 
  return $r;
 } 

 my $aa = read_csv_old($fname,$tdf);

 my $top_row_ref = $aa->[0];
 my @rows        = @{$aa};
 my $numrows     = $#rows + 1;
 my @top_row     = @{$top_row_ref};
 my $numvars     = $#top_row + 1;

 for (my $i=0;$i<$numvars;$i++) {
  $top_row[$i] = lc($top_row[$i]);
  my $row_is_numeric = "numeric";
  for (my $j=1;$j<$numrows;$j++) {
   my $datum = "__undef";
   if (defined($aa->[$j][$i])) {
    $datum = lc($aa->[$j][$i]);
    $datum =~ s/^"//;
    $datum =~ s/"$//;
   }
   if (!looks_like_number($datum)) {
    $row_is_numeric = "categorical";
   } 
   $self->{dm}{data}{$top_row[$i]}[$j-1] = $datum;
  }
  $self->{dm}{info}{$top_row[$i]}{type} = $row_is_numeric;
 }
 $self->{gb}{dirty} = 1;
 $r->{status} = "ok";
 $oo->disp(sprintf("(%d vars, %d obs)\n\n",$numvars,$numrows-1));
 if ($self->{gb}{using_gui}) {
  tk_updateVars($self->{dm});
 } 
 return $r;
}



sub outsheet {
 my $self   = shift;
 my $a      = shift;
 my $argref = $a->{args};

 my $fname = undef;
 if ($a->{have_using}) {
  $fname = $a->{using};
 }
 my $r = {};
 my $oo = $self->{oo};

 if (!defined($fname)) {
  $r->{status} = "error -- no file name provided; usage: outsheet using <fname>";
  return $r;
 }

 my $os = "";
 my $count = 0;
 my $first_one = 1;
 foreach my $var (keys %{$self->{dm}{data}}) {
  if ($first_one) {
   my @ary = @{$self->{dm}{data}{$var}};
   $count = $#ary + 1;
   $first_one = 0;
  }
  $os .= "$var,";
 }
 chop $os;
 $os .= "\n";

 for (my $i=0;$i<$count;$i++) {
  foreach my $var (keys %{$self->{dm}{data}}) {
   my $datum = $self->{dm}{data}{$var}[$i];
#   $datum =~ s/"/&quot;/g;
   if ($datum =~ /[,\s]/) {
    $os .= "\"$datum\",";
   } else {
    $os .= "$datum,";
   }
  }
  chop $os;
  $os .= "\n";
 }

 open(OFILE,"> " . $fname);
 print OFILE $os;
 close OFILE;
 $self->{gb}{dirty} = 0;
 return { status => "ok" }; 
}


sub getPtrToExcelOLE {
 my $Excel;
 eval { $Excel = Win32::OLE->GetActiveObject('Excel.Application')};
 die "Excel not installed" if $@;
 unless (defined $Excel) {
  $Excel = Win32::OLE->new('Excel.Application', sub {$_[0]->Quit;}) or
   die "Oops, cannot start Excel";
 }
 return $Excel;
}


sub loadExcelConstants {
 my $Excel = shift;
 my $consts = Win32::OLE::Const->Load($Excel);
 return $consts;
};



sub convertXLSToCSV{
 my $Excel    = shift;
 my $consts   = shift;
 my $iName    = shift;
 my $oName    = shift;

 $Excel->{DisplayAlerts} = 0;

 my $wbk = $Excel->Workbooks->Open({
            FileName => $iName
                       });

 my $ws  = $wbk->Worksheets(1);

 my %properties;
 $properties{"converted_by_MSExcel"} = 1;

 my $success =  
     $ws->SaveAs({
           FileFormat => $consts->{xlCSV},
           FileName   => $oName,
	   AddToMru   => 0,
	   CreateBackup => 0,
                });

 $wbk->Close();
 return $success;
}

sub closeExcelOLE {
 my $Excel= shift;
 $Excel->Quit;
};


sub insheet_helper {
 my $self  = shift;
 my $oo    = shift;
 my $a     = shift;
 my $fname = shift;

 my $r   = { status => "ok" };
 my $tdf = tstFlg($a,"tab");
 my $xls = tstFlg($a,"xls");

 if (isUrl($fname)) {
 } else {
  if (!(-e $fname)) {
   $r->{status} = "fail; could not find file $fname";
   return $r;
  } 
 }

 if ($xls && isUrl($fname)) {
  $r->{status} = "fail; xls option cannot be combined with internet download";
  return $r;
 }

 if ($xls) {
  if ($self->{gb}{platform} eq "MSWin32") {
   my $E = $self->{ExcelOLE};
   my $C = $self->{ExcelOLEConstants};
   if (!defined($E)) {
    $oo->disp("Creating MS Excel OLE connection.\n");
    $E = getPtrToExcelOLE();
    $C = loadExcelConstants($E);
   }
   if (!defined($E)) {
    $r->{status} = "fail; could not open Excel OLE connection; Excel not installed?";
    return $r;
   }

   $tdf = 0;
   my $orig_fname = $fname;
   $fname = "$fname.$$.csv.tmp";
   $oo->disp("Creating temp conversion file \"$fname\".\n");
   my $ok = convertXLSToCSV($E,$C,$orig_fname,$fname);
   closeExcelOLE($E);
   $oo->disp("Closed MS Excel OLE connection.\n");

  } else {
   $r->{status} = "fail; Excel conversion only hosted on Windows";
   $r->{platform} = $self->{gb}{platform};
   return $r;
  }
 }
 my ($names, $aa) = read_csv($fname,$tdf);
 if ($xls) {
  unlink $fname;
 }

# print "insheet_helper: fname was $fname\n";
# print Dumper $r;
 return ($r,$names,$aa);
}


1;

