#!/usr/bin/perl -w

#   Copyright (C) 2006 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

use fgroup;
package fgroup::inxpt;
BEGIN { @ISA = qw(fgroup); };

use Exporter 'import';
@EXPORT_OK = qw();

use strict;
use gui qw(tk_updateVars);
use filewrap qw(open_read isUrl);
use xport;

sub commands {
 my $self = shift;
 my @cmds = ("inxpt");
 return @cmds;
}

sub inxpt {
 my $self   = shift;
 my $a      = shift;

 if ($self->{gb}{dirty}) {
  return { status => "fail; No. Data would be lost. Save or clear first." }
 }

 my $r = { status => "ok" };
 my $fn = "";
 if (!$a->{have_using}) {
  return { status => "fail; missing using val" }
 }
 $fn = $a->{using};

 if (isUrl($fn)) {
 } else {
  if (!(-e $fn)) {
   return { status => "fail; xpt file \"$fn\" not found" }
  }
 }

 my $xp = new xport;
 my ($ok,$fh) = open_read($fn);
 binmode($fh);
 if (!$ok) {
  return { status => "could not open file \"$fn\" for read" };
 }
 $xp->setFH($fh);
 my $xpt_data = $xp->parse();
 my $lib  = (values %{$xpt_data->{libraries}})[0];
 my $ds   = (values %{$lib->{datasets}})[0];

 my @varnames = map(lc($_->{nname}), @{$ds->{vars}});
 my @vartypes = map($_->{ntype}, @{$ds->{vars}});

 foreach my $i (0 .. $#varnames) {
  my $vn = $varnames[$i];
  my $vt = $vartypes[$i];
  $self->{dm}{info}{$vn}{type} = ($vt == 1) ? "numeric" : "categorical";
 }

 my $vdata = $ds->{data};

 foreach my $i (0 .. @{$vdata}-1) {
  foreach my $j (0 .. $#varnames) {
   my $vn = $varnames[$j];
   my $vd = $vdata->[$i][$j];
   # print "$vn $vd\n";
   $self->{dm}{data}{$vn}[$i] = $vd;
  }
 }
 $r->{status} = "ok";
 $r->{obscount} = scalar @{$vdata};
 $r->{varcount} = scalar @varnames;
 $self->{gb}{dirty} = 1;
 my $oo = $self->{oo};
 $oo->disp(sprintf("(%d observations read, %d variables)\n",$r->{obscount},$r->{varcount}));

 if ($self->{gb}{using_gui}) {
  tk_updateVars($self->{dm});
 }
 return $r;
};

1;

