#!/usr/bin/perl -w

#   Copyright (C) 2006 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

use fgroup;
package fgroup::iv_2sls_lr;
BEGIN { @ISA = qw(fgroup); };

use Exporter 'import';
@EXPORT_OK = qw(new);
use warnings 'all';

use Data::Dumper;
use misc qw(deep_copy);
use dataqual qw(gen_numquald_ha);
use fgroup::ols_lr qw(regress_pretty_print calc_ols_WaldF);

use misc2 qw(inDict tstFlg func_fail);

use gui qw(tk_updateVars);
use matrix_ols qw(m_2sls_regress m_2sls_robust);
use matrix_help qw(m_gen_sqrt_diag_a m_mean_tot_col_s m_ss_col_s a_gen_matrix_from_cols_m m_get_elem_s de_objify gen_zero_matrix_m m_set_elem);
# use Statistics::Distributions;
use mod_distributions;
use Statistics::Distributions;

use strict;

sub commands {
 my $self = shift;
 my @cmds = ("ivreg");
 return @cmds;
}

sub ivreg {
 my $self      = shift;
 my $a         = shift;
 my $argref    = $a->{args};
 my $oo        = $self->{oo};

 my $r = { status => "ok" };

 if (defined($a->{options}{aweight})) {
  $oo->disp("\nSorry, but no weighting is currently unimplemented for IV regression.\n");
  return { status => "fail; option [aweight=...] has not been implemented :(" };
 }
 $oo->tdisp('error',"\nWARNING WARNING WARNING");
 $oo->tdisp('error',"\n-----------------------\n");
 $oo->tdisp('error',"ivreg is *not* really tested ... garbage may follow\n");

 if (tstFlg($a,"robust")) {
  $oo->tdisp('error',"\"robust\" is even more experimental than ivreg!\n");
 }

 $oo->disp("\n");
 $r = $self->parseArgsIV($a);
 if (func_fail($r)) { return $r; };

 my $depvar     = $r->{depvar};
 my @varlist1   = @{$r->{independent}};
 my @varlist2   = @{$r->{instrumented}};
 my @varlist_iv = @{$r->{instrumental}};

 my @X_vars     = (@varlist1, @varlist2);
 my @Z_vars     = (@varlist1, @varlist_iv);

# print "X_vars = "; foreach my $x (@X_vars) { print $x . " "; }; print "\n";
# print "Z_vars = "; foreach my $z (@Z_vars) { print $z . " "; }; print "\n";

 my $X_count    = @X_vars;
 my $Z_count    = @Z_vars;

 my $av_h = {};
 foreach my $v (@varlist1, @varlist2, @varlist_iv, $depvar) {
  $av_h->{$v} = 1;
 }
 my @av = sort keys %{$av_h};

 my $qdm = {};
 $qdm = gen_numquald_ha($self->{dm},\@av, $a->{ifary});

 if (defined($self->{gb}{last_run})) {
  $self->{gb}{last_last_run} = deep_copy($self->{gb}{last_run});
 }
 delete $self->{gb}{last_run};
 $self->{gb}{last_run}{orig_from}  = deep_copy($qdm->{orig_from});
 $self->{gb}{last_run}{orig_depth} = $qdm->{orig_depth};

 my $X_ref_a = [];
 foreach my $xv ( @X_vars ) {
  push (@{$X_ref_a},$qdm->{data}{$xv});
 }

 my $Z_ref_a = [];
 foreach my $zv ( @Z_vars ) {
  push (@{$Z_ref_a},$qdm->{data}{$zv});
 }

 my $obs_count = @{$X_ref_a->[0]};

 my @ones_array = ();
 foreach my $row (@{$X_ref_a->[0]}) {
  push(@ones_array,1.0);
 }

 my $dv_a = $qdm->{data}{$depvar};
 $X_ref_a =  [ \@ones_array, @{$X_ref_a} ];
 $Z_ref_a =  [ \@ones_array, @{$Z_ref_a} ];

 my $dv_v = a_gen_matrix_from_cols_m([$dv_a]);
 my $X_m  = a_gen_matrix_from_cols_m($X_ref_a);
 my $Z_m  = a_gen_matrix_from_cols_m($Z_ref_a);

 my $disp_m    = undef;
 my $soln_m    = undef;
 my $resid_v   = undef;
 my $pred_v    = undef;
 my $rss       = undef;
 if (tstFlg($a,"robust")) {
  ($disp_m, $soln_m, $pred_v, $resid_v, $rss) = m_2sls_robust($X_m,$Z_m,$dv_v);
 } else {
  ($disp_m, $soln_m, $pred_v, $resid_v, $rss) = m_2sls_regress($X_m,$Z_m,$dv_v);
 }

 if (!defined($disp_m)) {
  return { status => "fail; could not create dispersion matrix; collinearity problem?" }
 }

 $self->{gb}{last_run}{type}              = tstFlg($a,"robust") ? "2sls_robust" : "2sls";
 $self->{gb}{last_run}{matrices}{ev_m}    = $X_m;
 $self->{gb}{last_run}{matrices}{iv_m}    = $Z_m;
 $self->{gb}{last_run}{matrices}{dv_v}    = $dv_v;
 $self->{gb}{last_run}{matrices}{disp_m}  = $disp_m;
 $self->{gb}{last_run}{matrices}{soln_m}  = $soln_m;
 $self->{gb}{last_run}{matrices}{pred_v}  = $pred_v;
 $self->{gb}{last_run}{matrices}{resid_v} = $resid_v;
 $r->{indeps}                             = \@X_vars;
 $self->{gb}{last_run}{indeps}            = \@X_vars;

 my $ses = m_gen_sqrt_diag_a($disp_m);

 my $i = 1;
 foreach my $ev ( ("_cons", @X_vars) ) {
  my $coeff = m_get_elem_s($soln_m,$i,1);
  $r->{coeffs}{$ev} = $coeff;
  $self->{gb}{last_run}{coeffs}{$ev} = $coeff;
  $i++;
 }

 $i = 0;
 foreach my $ev ( ("_cons", @X_vars) ) {
  my $stderr = $ses->[$i];
  $r->{stderrs}{$ev} = $stderr;
  $self->{gb}{last_run}{stderrs}{$ev}  = $stderr;
  $r->{t_scores}{$ev}              = $r->{coeffs}{$ev} / $stderr;
  $i++;
 }

 # predicted calc
 (my $actmean, my $acttot) = m_mean_tot_col_s($self->{gb}{last_run}{matrices}{dv_v},1);

 my $tss = m_ss_col_s($dv_v,$actmean,1);
 # my $ess = m_ss_col_s($pred_v,$actmean,1);
 my $ess = $tss-$rss;
 $r->{tss}              = $tss;
 $r->{ess}              = $ess;
 $r->{rss}              = $rss;
 $self->{gb}{last_run}{tss} = $tss;
 $self->{gb}{last_run}{ess} = $ess;
 $self->{gb}{last_run}{rss} = $rss;

 # Basic stats
 ##########################################################
 my $N    = @{$dv_a};
 my $k    = @{$X_ref_a} - 1;

 my $rsquared = $ess/$tss;
 $r->{rsquared}              = $rsquared;
 $self->{gb}{last_run}{rsquared} = $rsquared;

 my $mst   = $tss / ($N-1);
 my $mse   = $ess / ($k);
 my $msr   = $rss / ($N-($k+1));
 $r->{mst} = $mst;
 $r->{mse} = $mse;
 $r->{msr} = $msr;
 $self->{gb}{last_run}{mst} = $mst;
 $self->{gb}{last_run}{mse} = $mse;
 $self->{gb}{last_run}{msr} = $msr;

 $r->{k}   = $k;
 $r->{N}   = $N;
 $self->{gb}{last_run}{k}   = $k;
 $self->{gb}{last_run}{N}   = $N;

 my $t_crit = Statistics::Distributions::tdistr(($N-($k+1)),
	                                        ((1-$self->{gb}{conf})/2));
 $r->{t_crit}              = $t_crit;
 $self->{gb}{last_run}{t_crit} = $t_crit;

 my $sigma_e    = sqrt($msr);
 $r->{sigma_e}                 = $sigma_e;
 $self->{gb}{last_run}{sigma_e}    = $sigma_e;

 my $R_width = @X_vars + 1;
 my $R_depth = @X_vars;
 my $R_m = gen_zero_matrix_m($R_depth,$R_width);
 for (my $p=0;$p<@X_vars;$p++) {
  m_set_elem($R_m, $p+1,$p+2,1.0);
 }
 my $r_m = gen_zero_matrix_m($R_depth,1);
 my $tr = calc_ols_WaldF($self->{gb},$R_m,$r_m);
 if (func_fail($tr)) { 
  return $tr;
 } 
 my $F = $tr->{F};
 my $F_prob = $tr->{Prob_F};

 $r->{F}                   = $F;
 $self->{gb}{last_run}{F}      = $F;
 $r->{F_prob}              = $F_prob;
 $self->{gb}{last_run}{F_prob} = $F_prob;

 $oo->disp(" Instrumental Variables (2SLS) regression");
 if (!tstFlg($a,"robust")) { $oo->disp("\n"); };
 regress_pretty_print($self->{gb},$oo,$a,$r);

 my $os = "";
 $os .= "-----------------------------------------------------------------------------\n";
 $os .= "Instrumented :   ";
 foreach my $v (@varlist2) { $os .= "$v "; };
 $os .= "\nInstruments :    ";
 foreach my $v (@varlist1, @varlist_iv) { $os .= "$v "; };
 $os .= "\n";
 $os .= "-----------------------------------------------------------------------------\n";
 $os .= "(Note: r-squared and \"Model\" SS values are meaningless; ignore them)\n\n";

 $oo->disp($os);
 return $r;
};



sub parseArgsIV {
 my $self = shift;
 my $a    = shift;

 my @indeps        = ();
 my @instrumented  = ();
 my @instrumental  = ();
 # x1 x2 x2 ( x4 x5 = x6 x7 ) x8 
 my $ivs_open = 0;
 my $ivds     = 0;
 my $d1       = 0;
 my $r        = { status => "ok" };

 foreach my $arg (@{$a->{args}}) {
  if ($d1 || ($arg=~ /if/)) {
   $d1 = 1;
  } else {
   if ($ivs_open) {
    if ($ivds) {
     if ($arg =~ /\)$/) {
      $ivs_open= 0;
      $ivds= 0;
      $arg =~ s/\)$//;
      if (length($arg)) { push(@instrumental, $arg); };
     } else {
      push(@instrumental, $arg);
     }
    } else {
     if ($arg =~ /=/) {
      $ivds= 1;
      my ($a1,$a2) = split(/=/,$arg);
      if (defined($a1) && length($a1)) { push(@instrumented, $a1); };
      if (defined($a2) && length($a1)) { push(@instrumental, $a1); };
     } else {
      push(@instrumented, $arg);
     }
    }
   } else {
    if ($arg =~ /^\(/) {
     $ivs_open = 1;
     $ivds     = 0;
     $arg =~ s/^\(//;
     #    print "h1 ARG $arg\n";
     if (length($arg)) { push(@instrumented,$arg); };
    } else {
     push(@indeps,$arg);
    }
   }
  }
 };

 foreach my $ar (\@indeps, \@instrumented, \@instrumental) {
  foreach my $arg (@{$ar}) {
   if (!inDict($arg, $self->{dm}{data})) {
    return { status => "fail; unknown variable $arg" };
   }
  }
 }

 my $depvar = shift @indeps;

 if (!inDict($depvar,$self->{dm}{data})) {
  return { status => "fail; unknown dependent variable" };
 }

 if (!@indeps) {
  return { status => "fail; no independent variables specified" };
 }

 if (!@instrumental) {
  return { status => "fail; no instrumental variables specified. Maybe you want to run \"regress\" insteadl." };
 }

 $r->{independent}  = \@indeps;
 $r->{instrumental} = \@instrumental;
 $r->{instrumented}  = \@instrumented;
 $r->{depvar}        = $depvar;
 return $r;
};


1;

