#!/usr/bin/perl -w

#   Copyright (C) 2006 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

use fgroup;
package fgroup::simple_stat;
BEGIN { @ISA = qw(fgroup); };

use Exporter 'import';
@EXPORT_OK = qw(format_sum_line);

use misc qw(one_var_stats fixedFloat fixedFloat_p numCount min max);
use dataqual qw(gen_numquald_aa);

use misc2 qw(inAry tstFlg inDict func_fail);

use eqn_string qw(create_evaluable_eqn);

use Statistics::Distributions qw(tdistr tprob);
use Scalar::Util qw(looks_like_number);
use Data::Dumper;


sub commands {
 my $self = shift;
 my @cmds = ("sum", "summary", "ci", "ttest");
 return @cmds;
}

sub ci  {
 my $self    = shift;
 my $a       = shift;
 $a->{flag}{ci} = 1;
 return $self->sum($a);
};


sub summary {
 my $self = shift;
 my $a    = shift;
 return($self->sum($a));
}

sub sum {
 my $self    = shift;
 my $a       = shift;

 my $r = {};
 my $oo      = $self->{oo};

 my @vars = ();
 if ($a->{firstvar}{present}) {
#  print Dumper($a);
  foreach my $z (@{$a->{varary}}) {
   if ($z->{state} eq "exist") { 
    push(@vars, $z->{name}); 
   } else { 
    $r->{status} = "fail; unknown variable";
    return $r;
   };
  }
#  exit(1);
 }
 if (!(@vars)) {
  @vars = keys %{$self->{dm}{data}};
 }

 foreach my $var (@vars) {
  if (!defined($self->{dm}{data}{$var})) {
   $r->{status} = "fail; included unknown variable $var";
   return $r;
  }
 }

 if (tstFlg($a,"detail")) {
  my $os = "\n";
  my $r = undef;
  foreach my $var (@vars) { 
   $os .= "                       " . $var . "\n";
   $os .= "----------------------------------------------------------\n";
   my $qaa = gen_numquald_aa($self->{dm},[ $var ], $a->{ifary});
   my $qa  = $qaa->[0];
   $r   = one_var_stats($qa);
   if (!func_fail($r) && (scalar @{$qa})) {
    $os .= sprintf("%10.10s = %10.10s\n","Obs",$r->{count});
    $os .= sprintf("%10.10s = %10.10s\n","Mean",fixedFloat($r->{mean}));
    $os .= sprintf("%10.10s = %10.10s\n","Variance",fixedFloat_p($r->{variance}));
    $os .= sprintf("%10.10s = %10.10s\n","Std. Dev.",fixedFloat_p($r->{std_dev}));
    $os .= sprintf("%10.10s = %10.10s\n","Skewness",fixedFloat($r->{skewness}));
    $os .= sprintf("%10.10s = %10.10s\n","Kurtosis",fixedFloat_p($r->{kurtosis}));
    $os .= "\n";
    $os .= sprintf("%10.10s = %10.10s\n","min",fixedFloat($r->{min}));
    $os .= sprintf("%10.10s = %10.10s\n","max",fixedFloat($r->{max}));
    $os .= "\n";
    $os .= sprintf("0.1th  percentile = %10.10s\n",fixedFloat($r->{p001}));
    $os .= sprintf("  1st  percentile = %10.10s\n",fixedFloat($r->{p01}));
    $os .= sprintf("  5th  percentile = %10.10s\n",fixedFloat($r->{p05}));
    $os .= sprintf(" 10th  percentile = %10.10s\n",fixedFloat($r->{p10}));
    $os .= sprintf(" 25th  percentile = %10.10s\n",fixedFloat($r->{p25}));
    $os .= sprintf("\n median         = %10.10s\n\n",fixedFloat($r->{median}));
    $os .= sprintf(" 75th  percentile = %10.10s\n",fixedFloat($r->{p75}));
    $os .= sprintf(" 90th  percentile = %10.10s\n",fixedFloat($r->{p90}));
    $os .= sprintf(" 95th  percentile = %10.10s\n",fixedFloat($r->{p95}));
    $os .= sprintf(" 99th  percentile = %10.10s\n",fixedFloat($r->{p99}));
    $os .= sprintf("99.8th percentile = %10.10s\n",fixedFloat($r->{p999}));
    $os .= "\n";
   } else {
    $os .= "Var $var fails. message: \"$r->{status}\"\n";
   }
   $oo->disp($os); $os = "";
  }
  $os .= "\n";
  $oo->disp($os);
  return $r;
 } else {
  my $os = "\n";
  if (tstFlg($a,"ci")) {
   $os .= sprintf("Variable   |    Obs       Mean   Std. Dev. [ %2d%% Conf Interval]\n",$self->{gb}{conf}*100);
   $os .= "-----------+--------------------------------------------------------\n";
  } else {
   $os .= "Variable   |    Obs       Mean  Std. Dev.       Min         Max\n";
   $os .= "-----------+--------------------------------------------------------\n";
  }

  foreach my $var (sort @vars) {
   my $qaa = gen_numquald_aa($self->{dm},[ $var ],$a->{ifary});
   my $qary = $qaa->[0];
   if (scalar @{$qary}) {
    $r = one_var_stats($qary);
   } else {
    $r = { status => "fail; no numeric observations" };
   }
   if (func_fail($r)) { 
    $os .= sprintf("%10s | %s", $var, $r->{status});
   } else {
    $os .= format_sum_line($var,$self->{gb},$r,$a);
   }
   $oo->disp($os . "\n"); $os="";
  }
  $os .= "\n";
  $oo->disp($os);
 };

 $r->{status} = "ok";
 return $r;
}


sub ttest_parse_args {
 my $self    = shift;
 my $a       = shift;
 my $argref  = $a->{args};
 my $flagref = $a->{flgs};
 my $r       = { status => "ok" };

 my @vars    = @{$argref};
 my @flags   = @{$flagref};

 # scenarios:
 # ttest a, by(c) [options]
 # ttest a if b, by(c) [options]
 # ttest a = b [,options]
 # ttest a = b if c [,options];

 my $have_byvar  = 0;
 my $have_1stvar = 0;
 my $have_2ndvar = 0;
 my $have_if     = 0;
 my $var1        = "";
 my $var2        = "";
 my $unpaired    = 0;
 my $unequal     = 0;

 if (!(@vars)) {
  return { status => "fail; no arguments" };
 }
 $var1 = shift @vars;
 if (!inDict($var1,$self->{dm}{data})) {
  return { status => "fail; unknown var ($var1)" };
 } else {
  $have_1stvar = 1;
 }

 if (!(@vars)) {
  # no more pre-comma stuff, go to by section
  goto flag_section;
 } else {
  my $t = shift @vars;
  if ($t eq "if") {
   my $eqn_str = "";
   while (@vars) { $eqn_str .= (shift @vars) . " "; }
   $r = create_evaluable_eqn($self->{dm},$self->{gb},$eqn_str,1);
   if (func_fail($r)) { return $r; }
   $have_if = 1;
  } elsif ($t eq "==") {
   $var2 = shift @vars;
   if (!inDict($var2,$self->{dm}{data})) {
    return { status => "fail; unknown second var\"$t\"" };
   }
   $have_2ndvar = 1;
   if (@vars) {
    my $t = shift @vars;
    if ($t eq "if") {
     my $eqn_str = "";
     while (@vars) { $eqn_str .= (shift @vars) . " "; }
     $r = create_evaluable_eqn($self->{dm},$self->{gb},$eqn_str,1);
     if (func_fail($r)) { return $r; }
     $have_if = 1;
    }
   }
  } else {
   return { status => "fail; garbled command. what is \"$t\"" };
  }
 }

 flag_section:
 while (@flags) {
  my $t = shift @flags;
  # print "flag $t\n";
  if ($t =~ /by\(([a-z]\w*)\)/) {
   $byvar = $1;
   if (!inDict($byvar,$self->{dm}{data})) {
    return { status => "fail; unknown \"by\" var" };
   }
   $have_byvar = 1;
  } elsif ($t eq "unpaired") {
   $unpaired = 1;
  } elsif ($t eq "unequal") {
   $unequal= 1;
  }
 }

 if ($have_byvar && $have_2ndvar) {
  $r->{status} = "fail; specified \"by\" variable and compare variable. only one or the other acceptable";
  return $r;
 }

 $r->{var1}        = $var1;
 $r->{have_byvar}  = $have_byvar;
 if ($have_byvar) {
 $r->{byvar}       = $byvar;
 } else {
 $r->{var2}        = $var2;
 }
 $r->{have_2ndvar} = $have_2ndvar;
 $r->{have_if}     = $have_if;
 if ($have_if) {
  $r->{ifexp}       = $r->{expression};
 }
 $r->{unpaired}    = $unpaired;
 $r->{unequal}     = $unequal;

 return $r;
};




sub ttest {
 my $self    = shift;
 my $a       = shift;
 my $argref  = $a->{args};
 my $flagref = $a->{flgs};
 my $r = {};

 $r = $self->ttest_parse_args($a);
 if (func_fail($r)) { return $r; }

 if (0) {
  foreach my $k (keys %{$r}) {
   print "key $k val " . $r->{$k} . "\n";
  }
 }

 $r = $self->calc_ttest($r);
 if (func_fail($r)) {
  return $r;
 } else {
  $self->show_ttest($r->{results},
	            $r->{unequal},
		    $r->{unpaired},
		    $r->{have_2ndvar});
  return $r;
 };
}


sub show_ttest {
 my $self     = shift;
 my $r        = shift;

 my $unequal     = shift;
 my $unpaired    = shift;
 my $have_2ndvar = shift;

 my $oo = $self->{oo};
 my $os = "\n\n";

 if ($have_2ndvar) { 
  if (!$unpaired) { $os .= "Paired "; }
  else { $os .= "Unpaired "; }
 } 
 if ($unpaired) { $os .= "Unpaired "; };
 $os .= "Two sample t test with ";
 if ($unequal) { $os .= "unequal "; }
 else { $os .= "equal "; }
 $os .= "variances\n";

 $os .= "------------------------------------------------------------------------------\n";
 $os .= "   Group  |   Obs        Mean  Std. Err.  Std. Dev.  [95% Conf. Interval]\n";
 $os .= "----------+-------------------------------------------------------------------\n";
 my @binnames = (); 
 foreach my $bn (sort keys %{$r}) {
  push (@binnames,$bn) unless ($bn =~ /^__/);
 }
 my $bin0  = $binnames[0];
 $os .= sprintf(" %8.8s | %6d %s %s %s %s %s\n",
                $bin0,
		$r->{$bin0}{count},
		fixedFloat($r->{$bin0}{mean}),
		fixedFloat($r->{$bin0}{std_err}),
		fixedFloat($r->{$bin0}{std_dev}),
		fixedFloat($r->{$bin0}{mean} - $r->{$bin0}{t_crit}*$r->{$bin0}{std_err}),
		fixedFloat($r->{$bin0}{mean} + $r->{$bin0}{t_crit}*$r->{$bin0}{std_err}));
 my $bin1  = $binnames[1];
 $os .= sprintf(" %8.8s | %6d %s %s %s %s %s\n",
                $bin1,
		$r->{$bin1}{count},
		fixedFloat($r->{$bin1}{mean}),
		fixedFloat($r->{$bin1}{std_err}),
		fixedFloat($r->{$bin1}{std_dev}),
		fixedFloat($r->{$bin1}{mean} - $r->{$bin1}{t_crit}*$r->{$bin1}{std_err}),
		fixedFloat($r->{$bin1}{mean} + $r->{$bin1}{t_crit}*$r->{$bin1}{std_err}));
 if (!$have_2ndvar || $unpaired) {
 $os .= "----------+-------------------------------------------------------------------\n";
 my $cmb  = "__combined";
 $os .= sprintf(" %8.8s | %6d %s %s %s %s %s\n",
                "combined",
		$r->{$cmb}{count},
		fixedFloat($r->{$cmb}{mean}),
		fixedFloat($r->{$cmb}{std_err}),
		fixedFloat($r->{$cmb}{std_dev}),
		fixedFloat($r->{$cmb}{mean} - $r->{$cmb}{t_crit}*$r->{$cmb}{std_err}),
		fixedFloat($r->{$cmb}{mean} + $r->{$cmb}{t_crit}*$r->{$cmb}{std_err}));
 };
 $os .= "----------+-------------------------------------------------------------------\n";
 if ($have_2ndvar && !$unpaired) {
 $os .= sprintf(" %8.8s | %6d %s %s %s %s %s\n",
                "diff",
		$r->{__diff}{count},
		fixedFloat($r->{__diff}{mean}),
		fixedFloat($r->{__diff}{std_err}),
		fixedFloat($r->{__diff}{std_dev}),
		fixedFloat($r->{__diff}{mean} - $r->{__diff}{std_err}*$r->{__diff}{t_crit}),
		fixedFloat($r->{__diff}{mean} + $r->{__diff}{std_err}*$r->{__diff}{t_crit}));
 } elsif ($have_2ndvar && $unpaired) {
 $os .= sprintf(" %8.8s | %6d %s %s            %s %s\n",
                "diff",
		$r->{__diff}{count},
		fixedFloat($r->{__diff}{mean}),
		fixedFloat($r->{__pooled}{std_err}),
		fixedFloat($r->{__diff}{mean} - $r->{__pooled}{std_err}*$r->{__pooled}{t_crit}),
		fixedFloat($r->{__diff}{mean} + $r->{__pooled}{std_err}*$r->{__pooled}{t_crit}));
 } else {
 my $meandiff = $r->{$bin0}{mean} - $r->{$bin1}{mean};
 $os .= sprintf(" %8.8s |        %s %s            %s %s\n",
                "diff",
		fixedFloat($meandiff),
		fixedFloat($r->{__pooled}{std_err}),
		fixedFloat($meandiff - $r->{__pooled}{std_err}*$r->{__pooled}{t_crit}),
		fixedFloat($meandiff + $r->{__pooled}{std_err}*$r->{__pooled}{t_crit}));
 }
 $os .= "------------------------------------------------------------------------------\n";
 my $t_score = 999;
 if ($have_2ndvar && !$unpaired) {
 $os .= "    diff = mean(" . $bin0 . "-" . $bin1 . ")      t=" . fixedFloat($r->{__pooled}{t_score}) . "\n";
 } else {
 $os .= "    diff = mean(" . $bin0 . ") - mean(" . $bin1 . ")      t=" . fixedFloat($r->{__pooled}{t_score}) . "\n";
 }
 my $d_o_f = $r->{__pooled}{dof};
 if ($self->{unequal}) {
  $os .= "    Ho: diff = 0             Satterthwaite's of freedom =" . fixedFloat($d_o_f) . "\n";
 } else {
 $os .= "    Ho: diff = 0                      degrees of freedom =" . fixedFloat($d_o_f) . "\n";
 }
 $os .= "    Ha: diff < 0                 Ha: diff != 0                 Ha: diff > 0\n\n";
 $os .= sprintf(" Pr(T < t) = %s   Pr(|T| > |t|) = %s     Pr(T > t) = %s\n",
	        fixedFloat($r->{__pooled}{T_lt_t}),
		fixedFloat($r->{__pooled}{T_eq_t}),
		fixedFloat($r->{__pooled}{T_gt_t}));

 $os .= "\n\n";
 my $stdout = $self->{stdout};
 
 $oo->disp($os);

 return $r;
}



sub calc_ttest {
 my $self    = shift;
 my $r       = shift;
 my $stdout  = $self->{stdout};

 my $xname = "";
 my $yname = "";
 my $xr = {};
 my $yr = {};
 my $tr = {};

 if ($r->{have_byvar}) {
  # gen two arrays from one variable using "by"
  my $bybins  = {};
  my $bybinct = {};
  my @bydata  = @{$self->{dm}{data}{$r->{byvar}}};
  for (my $row=0;$row<@bydata;$row++) {
   # print "doing row of bydata $row @bydata\n";
   if (($bydata[$row] ne "__undef") && 
       (!($r->{have_if}) || $r->{ifary}[$row]) &&
       (looks_like_number($self->{dm}{data}{$r->{var1}}[$row]))) {
    if (!defined($bybinct->{$bydata[$row]})) {
     $bybinct->{$bydata[$row]} = 0;
    } else {
     $bybinct->{$bydata[$row]} += 1
    }
    $bybins->{$bydata[$row]}[$bybinct->{$bydata[$row]}] =
     $self->{dm}{data}{$r->{var1}}[$row];
   }
  }

  my @bins = sort keys %{$bybins};
  #print "==== bins is @bins " . @bins . "\n";
  $self->{bins} = \@bins;
  if (@bins < 2) {
   return { status => "fail; \"by\" var ($byvar) has too few bins" };
  } elsif (@bins> 2) {
   return { status => "fail; \"by\" var ($byvar) has too many bins" };
  }
  $xname = $bins[0];
  $yname = $bins[1];
  $xr = one_var_stats($bybins->{$bins[0]});
  $yr = one_var_stats($bybins->{$bins[1]});
  my @combined_ary = (@{$bybins->{$bins[0]}}, @{$bybins->{$bins[1]}});
  $tr = one_var_stats(\@combined_ary);
 } else {
  # use two variables to create arrays;
  if (!inDict($r->{var1},$self->{dm}{data}) ||
      !inDict($r->{var2},$self->{dm}{data})) {
   return { status => "fail; specified variable not present" }
  }
  my @x = @{$self->{dm}{data}{$r->{var1}}};
  my @y = @{$self->{dm}{data}{$r->{var2}}};
  my @xx = ();
  my @yy = ();
  my @xx_minus_yy = ();
  my $rowcount = 0;
  for (my $row=0;$row<@x;$row++) {
   if (!($r->{have_if}) || $r->{ifary}[$row]) {
    $xx[$rowcount] = $self->{dm}{data}{$r->{var1}}[$row];
    $yy[$rowcount] = $self->{dm}{data}{$r->{var2}}[$row];
    $xx_minus_yy[$rowcount] = $xx[$rowcount] - $yy[$rowcount];
    $rowcount++;
   }
  }
  $xname = $r->{var1};
  $yname = $r->{var2};
  $xr = one_var_stats(\@xx);
  $yr = one_var_stats(\@yy);
  my @cmbary = (@xx,@yy);
  $tr = one_var_stats(\@cmbary);
  $dr = one_var_stats(\@xx_minus_yy);
  $r->{results}{__diff} = $dr;
 }

 $r->{results}{$xname} = $xr;
 $r->{results}{$yname} = $yr;
 $r->{results}{__combined} = $tr;

 foreach my $re (keys %{$r->{results}}) {
  if ($re =~ /status|__pooled/) {
  } else {
   $r->{results}{$re}{t_crit} = tdistr($r->{results}{$re}{count}-1,(1-$self->{gb}{conf})/2);
  }
 }

 my $dof = 0;
 my $pooled_variance = 0;
 my $pooled_tcrit = 0;

 if ($r->{unequal}) {
  $dof = min($xr->{count},$xr->{count}) -1 ;

  my $n1 = $xr->{count};
  my $n2 = $yr->{count};
  my $w1 = $xr->{variance} / $n1;
  my $w2 = $yr->{variance} / $n2;

  my $df_numer  = ($w1+$w2)**2;
  my $df_denom1 = ($w1**2) / ($n1-1);
  my $df_denom2 = ($w2**2) / ($n2-1);
  $dof = $df_numer / ($df_denom1 + $df_denom2);

  $pooled_variance = ($xr->{variance}/$xr->{count}) +
	                ($yr->{variance}/$yr->{count});

  $pooled_tcrit = tdistr(int($dof),(1-$self->{gb}{conf})/2);

 } else {
  $dof = $xr->{count} + $yr->{count} - 2;
  $pooled_variance =
   ((($xr->{count} - 1) * $xr->{variance}) +
    (($yr->{count} - 1) * $yr->{variance})) / $dof;
  $pooled_variance *= ( (1/$xr->{count}) + (1/$yr->{count}) );
  $pooled_tcrit = tdistr($dof,(1-$self->{gb}{conf})/2); 
 };

 my $pooled_se    = sqrt($pooled_variance);
 my $t_score      = 0;

 if ($r->{have_2ndvar}) {
  $dof          = $tr->{count}-2;
 }

 if ($r->{have_2ndvar} && !$r->{unpaired}) {
  $dof          = $dr->{count}-1;
  $t_score      = $dr->{mean} / $dr->{std_err};
 } else {
  $t_score      = ($xr->{mean} - $yr->{mean}) / $pooled_se;
 }

 $r->{results}{__pooled}{variance} = $pooled_variance;
 $r->{results}{__pooled}{std_err}  = $pooled_se;
 $r->{results}{__pooled}{dof}      = $dof;
 $r->{results}{__pooled}{t_crit}   = $pooled_tcrit;
 $r->{results}{__pooled}{t_score}  = $t_score;
 $r->{results}{__pooled}{T_lt_t}   = tprob(int($dof),-$t_score);
 $r->{results}{__pooled}{T_gt_t}   = tprob(int($dof), $t_score);
# $r->{results}{__pooled}{T_eq_t}   = max(1,tprob(int($dof),abs($t_score))*2);
 $r->{results}{__pooled}{T_eq_t}   = min(1,
	                                 2*min($r->{results}{__pooled}{T_lt_t},
	                                       $r->{results}{__pooled}{T_gt_t})
				        );

 return $r;
}

sub format_sum_line {
 my $var = shift;
 my $gb  = shift;
 my $r   = shift;
 my $a   = shift;
 my $os  = "";
 if (length($var) > 10) {
  while (length($var) > 10) {
   $os .= sprintf("%10s |\n",(substr($var,0,7,'') . "..."));
  }
  $os .= sprintf("%10s | ",$var);
 } else {
  $os .= sprintf("%10s | ",$var);
 }
 if (tstFlg($a,"ci")) {
  my $t;
  if ($r->{count} > 1) { 
   $t = tdistr(($r->{count})-1,1-($gb->{conf}));

   $lb = fixedFloat($r->{mean} - $t*$r->{std_dev});
   $ub = fixedFloat($r->{mean} + $t*$r->{std_dev});
  } else {
   $lb = "__undef    ";
   $ub = "__undef    ";
  }
  $os .= sprintf("%6d %s %s  %s %s",
         $r->{count},
         fixedFloat($r->{mean}),
         fixedFloat_p($r->{std_dev}),
         $lb,$ub,
         fixedFloat($r->{mean} + $t*$r->{std_dev}));
 } else {
  $os .= sprintf("%6d %s %s %s %s",
         $r->{count},
         fixedFloat($r->{mean}),
         fixedFloat_p($r->{std_dev}),
         fixedFloat($r->{min}),
         fixedFloat($r->{max}));
 }
 return $os;
}

1;

