#!/usr/bin/perl -w

#   Copyright (C) 2006-2008 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#



#perl2exe_include "Tk/Scrollbar.pm"
#perl2exe_include "Tk/Menu.pm"
#perl2exe_include "Tk/Menubutton.pm"
#perl2exe_include "Tk/BrowseEntry.pm"
#perl2exe_exclude "Win32Util.pm"
#perl2exe_include "Tk/arrowdownwin.xbm"

package gui;
use warnings 'all';

use Exporter 'import';
@EXPORT_OK = qw(tk_MainLoop setupGui setGuiCmdsRef tk_handleCommand tk_getOldCmd tk_updateVars register_menus tk_Insheet disableCmds enableCmds);

use strict;
use Data::Dumper;


use cmd_driver qw(handleCommand);

use     Tk;
use     Tk::FileSelect;
require Tk;
require Tk::Widget;
require Tk::ROText;
require Tk::Checkbutton;
require Tk::BrowseEntry;

my $tktree   = undef;
my $gb       = undef;
my $oo       = undef;
my $commands = undef;
my $dm       = undef;
my $result_width  = 80;
my $result_height = 30;
my $main_title    = "AT-ATS Statistics ";

# this is the current location of our "cursor" in the command
# history; that is; if we pageup/pagedown, through old command
# this is where we are
my $currCmdHist = 1;

sub entry_handleCommand {
 my $cmd = $gb->{tktree}{cmds}->get('1.0','end');
 $gb->{tktree}{cmds}->delete('1.0','end');
 chomp $cmd;
 $cmd =~ s/\s*$//;
 $cmd =~ s/\n//g;
 tk_handleCommand($cmd);
}

sub tk_handleCommand {
 my $cmd = shift;

 disableCmds();
 my $ok = handleCommand($commands,$oo,$gb,$dm,$cmd);
 if ($ok =~ /(^fail)|(^unknown)/) {
 } else {
  if ($cmd ne $gb->{tktree}{oldcmds}->get('end')) {
   $gb->{tktree}{oldcmds}->insert('end',$cmd);
   $gb->{tktree}{oldcmds}->see('end');
  }
  $currCmdHist = $gb->{tktree}{oldcmds}->size() - 1;
 }
 enableCmds();
}


sub tk_getOldCmd  {
 my $lb = shift;
 my $cs  = $lb->curselection();
 if (defined($cs) && ($cs ne "")) {
  my $old = $lb->get($lb->curselection());
  $gb->{tktree}{cmds}->delete('1.0','end');
  $gb->{tktree}{cmds}->insert('end',$old);
 }
}

sub  tk_EntryPrevCommand {
 my $cmd = shift;
 my $lb = $gb->{tktree}{oldcmds};
 my $cmdt = $lb->get($currCmdHist);
 $gb->{tktree}{cmds}->delete('1.0','end');
 $gb->{tktree}{cmds}->insert('end',$cmdt);
 $currCmdHist -= 1;
 if ($currCmdHist < 1) { $currCmdHist = 1; };
}

sub  tk_EntryNextCommand {
 my $cmd = shift;
 my $lb = $gb->{tktree}{oldcmds};
 my $cmdt = $lb->get($currCmdHist);
 $gb->{tktree}{cmds}->delete('1.0','end');
 $gb->{tktree}{cmds}->insert('end',$cmdt);
 $currCmdHist += 1;
 if ($currCmdHist > ($lb->size()-1)) { $currCmdHist = ($lb->size() -1); };
}

sub tk_getHelpCmdHyper {
 my $tg = shift;
 my $xy = shift;
 my $ind   = $gb->{tktree}{results}->index($xy);
 my @range = $gb->{tktree}{results}->tagPrevrange('help_cmd',$ind);
 my $text = $gb->{tktree}{results}->get(@range);
 $text =~ s/^\s*\.\s*//;
 $gb->{tktree}{cmds}->delete('1.0','end');
 $gb->{tktree}{cmds}->insert('end',$text);
}

sub tk_getHelpHelpHyper {
 my $tg = shift;
 my $xy = shift;
 my $ind   = $gb->{tktree}{results}->index($xy);
 my @range = $gb->{tktree}{results}->tagPrevrange('help_help',$ind);
 my $text = $gb->{tktree}{results}->get(@range);
 $text = "help $text";
 tk_handleCommand($text);
}

sub tk_getCmdHyper {
 my $tg = shift;
 my $xy = shift;
 my $ind   = $gb->{tktree}{results}->index($xy);
 my @range = $gb->{tktree}{results}->tagPrevrange('command',$ind);
 my $text = $gb->{tktree}{results}->get(@range);
 $text =~ s/^\s*\.\s*//;
 $gb->{tktree}{cmds}->delete('1.0','end');
 $gb->{tktree}{cmds}->insert('end',$text);
};


sub tk_copyVarCmd{
 my $lb = shift;
 my $cs = $lb->curselection();
 if (defined($cs) && ($cs ne "")) {
  my $var = $lb->get($cs);
  $gb->{tktree}{cmds}->insert('end'," " . $var);
 }
}

sub tk_MainLoop {
 $tktree->{cmds}->focusForce();
 MainLoop();
}

sub tk_updateVars {
 my $dm = shift;
 my @vars = sort keys %{$dm->{data}};
 if (defined($tktree)) {
  my $vlb = $gb->{tktree}{vars};
  $vlb->delete(0,'end');
  foreach my $v (@vars) {
   $vlb->insert('end',$v);
  }
  $vlb->see('end');
 }
}

sub setGuiCmdsRef {
 my $lcmds = shift;
 $commands = $lcmds;
}

sub disableCmds {
 my $cmd = $gb->{tktree}{cmds};
 my $main = $gb->{tktree}{main};
 $main->configure(-title=>"$main_title (BUSY)");
 $main->Busy(-recurse => 1);
 $cmd->configure(-state => 'disabled');
}

sub enableCmds{
 my $cmd = $gb->{tktree}{cmds};
 my $main = $gb->{tktree}{main};
 $main->Unbusy(-recurse => 1);
 $cmd->configure(-state => 'normal');
 $main->configure(-title=>"$main_title");
 $cmd->focus();
}

sub setupGui {
 my $loo       = shift;
 my $lgb       = shift;
 my $ldm       = shift;

 $dm = $ldm;
 $oo = $loo;
 $gb = $lgb;

 my $tk_main = MainWindow->new();
 $tk_main->title("$main_title");
 $tk_main->configure(-background => 'grey');

 # define some fonts to be used later
 $tk_main->fontCreate('C_bold',-family=>'courier',-size=>'10',-weight=>'bold');
 $tk_main->fontCreate('C_ital',-family=>'courier',-size=>'12',-slant=>'italic');
 $tk_main->fontCreate('C_helv',-family=>'helvetica',-size=>'10',-weight=>'bold');
 $tk_main->fontCreate('about_large',-family=>'times',-size=>'12',-weight=>'bold');
 $tk_main->fontCreate('help_title',-family=>'times',-size=>'12',-weight=>'bold');
 $tk_main->fontCreate('help_cmd',  -family=>'courier',-size=>'10',-weight=>'bold');

 # lay out all the static objects in the main window. This is messy, I know,
 # but what to do?
 my $tk_menu_bar = $tk_main->Frame(-relief=>'groove',-borderwidth=>3)->pack(-side=>'top',-fill=>'x');
 my $tk_main_bottom = $tk_main->Frame()->pack(-side=>'bottom',-fill=>'both',-expand=>'1');
 my $tk_main_bot_lt = $tk_main_bottom->Frame()->pack(-side=>'left',-fill=>'y');
 my $tk_main_bot_rt = $tk_main_bottom->Frame()->pack(-side=>'left',-expand=>'1',-fill=>'both');
 my $tk_results = $tk_main_bot_rt->Scrolled('ROText',-scrollbars=>'e',-width=>$result_width,-height=>$result_height,-wrap=>'word',-foreground=>'gray',-background=>'black')->pack(-expand=>'1',-fill=>'both',-side=>'top');
 my $tk_cmd_frm    = $tk_main_bot_rt->Frame()->pack(-side=>'bottom',-expand=>'0',-fill=>'x');
 my $tk_cmds_nm    = $tk_cmd_frm->Label(-text=>"Command:")->pack(-side=>'left');
 my $tk_cmds       = $tk_cmd_frm->Text(-font=>'C_helv',-width=>'80',-height=>'2')->pack(-side=>'right',-fill=>'x',-expand=>'1');
 my $tk_m_b_lt_t   = $tk_main_bot_lt->Frame()->pack(-side=>'top',-expand=>'1',-fill=>'y');
 my $tk_m_b_lt_b   = $tk_main_bot_lt->Frame()->pack(-side=>'top',-expand=>'1',-fill=>'y');
 my $tk_vars_lb    = $tk_m_b_lt_b->Scrolled('Listbox',-scrollbars => 'oe',-selectmode=>'single')->pack(-side=>'bottom',-expand=>'1',-fill=>'y');
 my $tk_vars_nm    = $tk_m_b_lt_b->Label(-text=>"Variables")->pack(-side=>'top');
 my $tk_oldcmd_nm  = $tk_m_b_lt_t->Label(-text=>"Review")->pack(-side=>'top');
 my $tk_oldcmd_lb  = $tk_m_b_lt_t->Scrolled('Listbox',-scrollbars=>'oe', -selectmode=>'single')->pack(-side=>'bottom',-fill=>'y',-expand=>'1');
 $tk_oldcmd_lb->insert('end',"");
 $tk_cmds->bind('<Return>', sub { entry_handleCommand(); } );
 $tk_cmds->bind('<Key-Prior>', sub { tk_EntryPrevCommand(); } );
 $tk_cmds->bind('<Key-Up>',    sub { tk_EntryPrevCommand(); } );
 $tk_cmds->bind('<Key-Next>',  sub { tk_EntryNextCommand(); } );
 $tk_cmds->bind('<Key-Down>',  sub { tk_EntryNextCommand(); } );
 $tk_oldcmd_lb->bind('<ButtonPress>', \&tk_getOldCmd );
 $tk_vars_lb->bind('<ButtonPress>', \&tk_copyVarCmd );

 $tk_results->tag('configure','error', -font=>'C_bold',-background=>'#201010',-foreground=>'red');
 $tk_results->tag('configure','warning', -font=>'C_bold',-background=>'#202000',-foreground=>'yellow');
 $tk_results->tag('configure','brite', -font=>'C_bold',-background=>'#000000',-foreground=>'yellow');
 $tk_results->tag('configure','banner', -background=>'#000000',-foreground=>'cyan');
 $tk_results->tag('configure','help_cmd', -font=>'C_bold',-foreground=>'white');
 $tk_results->tag('configure','help_help', -font=>'C_bold',-foreground=>'cyan');
 $tk_results->tag('configure','command', -font=>'C_ital',-background=>'#102010',-foreground=>'green');
 my $dummy = 7;
 $tk_results->tagBind('command','<ButtonPress>', [\&tk_getCmdHyper, Ev ('@')]);

 $tk_results->tagBind('command','<Any-Enter>' => sub {shift->tagConfigure('command',-relief=>'raised',-borderwidth=>'2') });
 $tk_results->tagBind('command','<Any-Leave>' => sub {shift->tagConfigure('command',-relief=>'flat',) });

 $tk_results->tagBind('help_cmd','<Any-Enter>' => sub {shift->tagConfigure('help_cmd',-relief=>'raised',-borderwidth=>'2',-background=>'#101010') });
 $tk_results->tagBind('help_cmd','<Any-Leave>' => sub {shift->tagConfigure('help_cmd',-relief=>'flat',-background=>'#000000') });
 $tk_results->tagBind('help_cmd','<1>', [\&tk_getHelpCmdHyper, Ev('@')]);

 $tk_results->tagBind('help_help','<Any-Enter>' => sub {shift->tagConfigure('help_help',-relief=>'raised',-borderwidth=>'2',-background=>'#101010') });
 $tk_results->tagBind('help_help','<Any-Leave>' => sub {shift->tagConfigure('help_help',-relief=>'flat',-background=>'#000000') });
 $tk_results->tagBind('help_help','<1>', [\&tk_getHelpHelpHyper, Ev('@')]);

 $tk_results->tag('configure','message', -foreground=>'blue');

 $oo->set_windowout($tk_results);
 $gb->{using_gui}           = 1;
 $gb->{tktree}{main}        = $tk_main;
 $gb->{tktree}{menubar}     = $tk_menu_bar;
 $gb->{tktree}{results}     = $tk_results;
 $gb->{tktree}{cmds}        = $tk_cmds;
 $gb->{tktree}{oldcmds}     = $tk_oldcmd_lb;
 $gb->{tktree}{vars}        = $tk_vars_lb;
}

sub register_menus {
 my $l_tktree  = shift;
 my $commands  = shift;
 $tktree       = $l_tktree;

 ##############
 # File
 my $tk_file_mb = $tktree->{menubar}->Menubutton(-text=>'File')->pack(-side=>'left');
 $tktree->{menus}{file} = $tk_file_mb;
 my $f_mb = $tktree->{menus}{file};
 $f_mb->AddItems(["command" => "Import Excel File", -command => \&tk_Insheet_Excel]);
 $f_mb->AddItems(["command" => "Import Comma Delimited File", -command => \&tk_Insheet_comma]);
 $f_mb->AddItems(["command" => "Import Tab   Delimited File", -command => \&tk_Insheet_tab]);
 $f_mb->AddItems(["command" => "Import Fixed Format File", -command => \&tk_Infix]);
 $f_mb->AddItems(["command" => "Save Comma Delimited File", -command => \&tk_Outsheet]);
 $f_mb->AddItems(["command" => "Execute \"do\" File", -command => \&tk_Do]);
 $f_mb->AddItems(["command" => "Clear Data", -command => \&tk_Clear]);
 $f_mb->AddItems(["command" => "Change Working Directory", -command => \&tk_Chdir]);
 $f_mb->AddItems(["command" => "Exit (w/o saving)", -command => \&tk_Exit]);

 ##############
 # Logging 
 my $logg_mb = $tktree->{menubar}->Menubutton(-text=>'Logging')->pack(-side=>'left');
 $tktree->{menus}{logging} = $logg_mb;
 $logg_mb->AddItems([command => "Open Log File", -command => \&tk_LogOpen]);
 $logg_mb->AddItems([command => "Close Log File", -command => \&tk_LogClose]);

 ##############
 # Info 
 my $info_mb = $tktree->{menubar}->Menubutton(-text=>'Info/Summary')->pack(-side=>'left');
 $tktree->{menus}{info} = $info_mb;
 $info_mb->AddItems([command => "Summary", -command => \&tk_Summary]);
 $info_mb->AddItems([command => "Tabulate One Var", -command => \&tk_Tab_One]);
 $info_mb->AddItems([command => "Tabulate Two Var", -command => \&tk_Tab_Two]);

 ##############
 # Regress 
 my $reg_mb = $tktree->{menubar}->Menubutton(-text=>'OLS Regression')->pack(-side=>'left');
 $tktree->{menus}{regress} = $reg_mb;
 $reg_mb->AddItems([command => "Regression", -command => \&tk_Regress]);
 $reg_mb->AddItems([command => "F-Test between Models", -command => \&tk_Exclusionrestrict]);

 ##############
 # Anova
 my $anova_mb = $tktree->{menubar}->Menubutton(-text => 'ANOVA')->pack(-side=>'left');
 $anova_mb->AddItems([command => "Oneway ANOVA", -command => \&tk_Anova_oneway ]);

 ##############
 # T-tests
 my $ttests_mb = $tktree->{menubar}->Menubutton(-text => 'T-Tests')->pack(-side=>'left');
 $ttests_mb->AddItems([command => "Compare 2 Variables", -command=> \&tk_Ttest_twovar ]);
 $ttests_mb->AddItems([command => "Compare Var A \"by\" Var B", -command=> \&tk_Ttest_onevar]);


 ##############
 # Graphs 
 my $graphs_mb = $tktree->{menubar}->Menubutton(-text => 'Graphs')->pack(-side=>'left');
 $graphs_mb->AddItems([command => "Histograms", -command=> \&tk_Histogram ]);
 $graphs_mb->AddItems([command => "Scatter Plots", -command=> \&tk_ScatterPlot ]);

 ##############
 # Help 
 my $help_mb = $tktree->{menubar}->Menubutton(-text => 'Help')->pack(-side=>'right');
 $help_mb->AddItems([command => "Help", -command => \&tk_Help ]);
 $help_mb->AddItems([command => "About", -command => \&tk_About ]);

}

sub tk_Exit {
 my $cmd = "quit, clear";
 tk_handleCommand($cmd);   
}

sub tk_Clear {
 my $cmd = "clear";
 tk_handleCommand($cmd);   
}

sub tk_Chdir {
 my $mw = $tktree->{main};
 my $tbor;
 my $newdir = "";
 my $e1;

 my $tk_Chdir_s0 = sub {
  $newdir = $tbor->chooseDirectory();
  $e1->delete(0,'end');
  $e1->insert(0, $newdir);
  $e1->xview('end');
 };

 my $tk_Chdir_s1 = sub {
  my $dir = $e1->get;
  if ($dir =~ /\s+/) {
   $dir = '"' . $dir . '"';
  }
  my $cmd = "cd " . $dir;
  $tbor->withdraw();
  tk_handleCommand($cmd);
 };

 my $tk_Chdir_s2 = sub {
  $tbor->withdraw();
 };

 if (!Exists($tbor)) {
  $tbor= $mw->Toplevel();
  $tbor->title("Change Working Directory");
  my $tf = $tbor->Frame()->pack(-side=>'top');
  my $mf = $tbor->Frame()->pack(-side=>'top');
  my $bf = $tbor->Frame()->pack(-side=>'top');
  my $l1 = $tf->Label(-text=>"Enter new Directory")->pack(-side=>'top');
  my $l2 = $mf->Label(-text=>"Dir:")->pack(-side =>'left');
     $e1 = $mf->Entry()->pack(-side=>'left');
  my $b1 = $mf->Button(-text => 'browse',-command=> $tk_Chdir_s0)->pack(-side=>'left');
  my $b2 = $bf->Button(-text=>'Go!',-command => $tk_Chdir_s1)->pack(-side=>'left');
  my $b3 = $bf->Button(-text=>'Cancel',-command => $tk_Chdir_s2)->pack(-side=>'right');
 }
}


sub tk_Insheet_comma {
 my $mw = $tktree->{main};
 my $types = [ ['Comma Delmited',   '.csv'           ],
               ['Text Files',       ['.txt', '.text']],
               ['dat',              '.dat',    'TEXT'],
               ['All Files',        '*',             ] ];
 my $file = $mw->getOpenFile(-filetypes => $types, -initialdir=> '.');
 if (defined($file) && length($file)) {
  tk_handleCommand("clear");
  my $cmd = "insheet using \"$file\", comma";
  tk_handleCommand($cmd);   
 } 
}

sub tk_Insheet_Excel {
 my $mw = $tktree->{main};
 my $types = [ ['Excel Workbook',   '.xls'           ],
               ['All Files',        '*',             ] ];
 my $file = $mw->getOpenFile(-filetypes => $types, -initialdir=> '.');
 if (defined($file) && length($file)) {
  tk_handleCommand("clear");
  my $cmd = "insheet using \"$file\", xls";
  tk_handleCommand($cmd);   
 } 
}

sub tk_Insheet_tab {
 my $mw = $tktree->{main};
 my $types = [ ['Tab Delmited',     '.tdf'           ],
               ['Text Files',       ['.txt', '.text']],
               ['dat',              '.dat',    'TEXT'],
               ['All Files',        '*',             ] ];
 
 my $file = $mw->getOpenFile(-filetypes => $types, -initialdir=> '.');
 if (defined($file) && length($file)) {
  tk_handleCommand("clear");
  my $cmd = "insheet using \"$file\", tab";
  tk_handleCommand($cmd);   
 } 
}

sub tk_Infix {
 my $mw = $tktree->{main};
 my $types = [ ['Dictionary',       '.dct'           ],
               ['dat',              '.dat',    'TEXT'],
               ['Text Files',       ['.txt', '.text']],
               ['All Files',        '*',             ] ];
 my $file = $mw->getOpenFile(-filetypes => $types, -initialdir=> '.');
 if (defined($file) && length($file)) {
  tk_handleCommand("clear");
  my $cmd = "infix using \"$file\"";
  tk_handleCommand($cmd);   
 } 
}

sub tk_Outsheet {
 my $mw = $tktree->{main};
 my $types = [ ['Comma Delmited',   '.csv'           ],
               ['Text Files',       ['.txt', '.text']],
               ['dat',              '.dat',    'TEXT'],
               ['All Files',        '*',             ] ];
 my $file = $mw->getSaveFile(-filetypes => $types, -initialdir=> '.');
 if (defined($file) && length($file)) {
  my $cmd = "outsheet using \"$file\"";
  tk_handleCommand($cmd);   
 } 
}

sub tk_LogOpen {
 my $mw = $tktree->{main};
 my $types = [ ['Log Files',         '.log',   'TEXT'],
               ['Text Files',       ['.txt', '.text']],
               ['Comma Delmited',   '.csv'           ],
               ['All Files',        '*',             ] ];
 my $file = $mw->getSaveFile(-filetypes => $types, -initialdir=> '.');
 if (defined($file) && length($file)) {
  my $cmd = "log using \"$file\"";
  tk_handleCommand($cmd);   
 } 
}

sub tk_Do {
 my $mw = $tktree->{main};
 my $types = [ ['Do Files',         ['.do', '.dont','TEXT']],
               ['All Files',        '*',             ] ];
 my $file = $mw->getOpenFile(-filetypes => $types, -initialdir=> '.');
 if (defined($file) && length($file)) {
  my $cmd = "do \"$file\"";
  tk_handleCommand($cmd);   
 } 
}

sub tk_LogClose {
 tk_handleCommand("log close");
}

sub tk_Summary {
 tk_handleCommand("sum");
}

sub tk_Tab_One {
 my $mw = $tktree->{main};

 my ($F1, $F2);
 my $LBL1;
 my $LB1;
 my ($BT1, $BT2);
 my $tbor;
 my $var;

 my $tk_Tab_One_s0 = sub {
  my $cmd = "tab $var";
  $tbor->withdraw();
  tk_handleCommand($cmd);
 };

 if (!Exists($tbor)) {
  $tbor= $mw->Toplevel();
  $tbor->title("One Variable Tabulate");
  $F1 = $tbor->Frame()->pack(-side => 'top');
  $LBL1 = $F1->Label(-text => 'Variable Selection')->pack(-side => 'top');
  $LB1  = $F1->BrowseEntry(-variable => \$var)->pack(-side => 'top');
  $F2   = $F1->Frame()->pack(-side => 'bottom');
  $BT1  = $F2->Button(-text => 'tabulate',-command => $tk_Tab_One_s0)->pack(-side => 'left');
  $BT2  = $F2->Button(-text => 'cancel',-command => sub { $tbor->withdraw(); })->pack(-side => 'right');
 } else {
  $tbor->deiconify();
  $tbor->raise();
 }

 my @vars = sort keys %{$dm->{data}};
 $LB1->delete(0,'end');
 foreach my $v (@vars) {
  $LB1->insert('end',$v);
 } 


}

sub tk_Tab_Two {
 my $mw = $tktree->{main};

 my $F1;
 my $F1t;
 my $F1b;
 my $F2l;
 my $F2r;
 my $LBL1;
 my $LBL2;
 my $LB1;
 my $LB2;
 my $F3l;
 my $F3r;
 my $BT1;
 my $CK1;
 my $tbor;
 my $chi2;

 my $var1; my $var2;
 my @vars = sort keys %{$dm->{data}};

 my $tk_Tab_Two_s0 = sub {
  my $cmd = "tab " . $var1 . " " .  $var2;
  if ($chi2) { $cmd .= ", chi2"; }
  $tbor->withdraw();
  tk_handleCommand($cmd);
 };

 if (!Exists($tbor)) {
  $tbor= $mw->Toplevel();
  $tbor->title("Two Variable Tabulate");
  $F1 = $tbor->Frame()->pack(-side => 'top');
  $F1t = $F1->Frame()->pack(-side => 'top');
  $F1b = $F1->Frame()->pack(-side => 'bottom');
  $F2l = $F1t->Frame()->pack(-side => 'left');
  $F2r = $F1t->Frame()->pack(-side => 'right');
  $LBL1 = $F2l->Label(-text => 'Variable One Select')->pack(-side => 'top');
  $LB1  = $F2l->BrowseEntry(-variable => \$var1)->pack(-side => 'bottom');
  $LBL2 = $F2r->Label(-text => 'Variable Two Select')->pack(-side => 'top');
  $LB2  = $F2r->BrowseEntry(-variable => \$var2)->pack(-side => 'bottom');
  $F3l  = $F1b->Frame()->pack(-side => 'left');
  $F3r  = $F1b->Frame()->pack(-side => 'right');
  $BT1  = $F3l->Button(-text => 'tabulate',-command => $tk_Tab_Two_s0)->pack(-side => 'bottom');
  $CK1  = $F3r->Checkbutton(-text => 'Perform chi2 test',-variable => \$chi2)->pack(-side=>'bottom');
 } else {
  $tbor->deiconify();
  $tbor->raise();
 }

 $LB1->delete(0,'end');
 $LB2->delete(0,'end');
 foreach my $v (sort @vars) {
  $LB1->insert('end',$v);
  $LB2->insert('end',$v);
 }
}

sub tk_Regress {
 my $mw = $tktree->{main};

 my $tbor;
 my @ind0;
 my @ind1;
 my $do_robust;
 my $do_weight;

 my @vars = sort keys %{$dm->{data}};

 my $dv;
 my $wv;
 my ($f1, $f2, $f3, $f4, $f5, $f6);
 my ($w0, $w1, $w2, $w3, $w4, $w5, $w6, $w7, $w8, $w9);

 my $ivs = "";
 my $tk_Regress_s0 = sub {
  foreach my $k (@ind1) {
   my $iv = $w5->get($k);
   $ivs .= $iv . " ";
  }
  my $cmd = "regress " . $dv . " " . $ivs;
  if ($do_weight) { $cmd .= "[aweight=$wv] "; }
  if ($do_robust) { $cmd .= ", robust"; }
  $tbor->withdraw();
  tk_handleCommand($cmd);
 };

 if (!Exists($tbor)) {
  $tbor= $mw->Toplevel();
  $tbor->title("OLS Regression");
  $f1  = $tbor->Frame()->pack(-side => 'top');
  $f2  = $f1->Frame()->pack(-side => 'top');
  $f3  = $f1->Frame()->pack(-side => 'bottom');
  $f4  = $f2->Frame()->pack(-side => 'left');
  $f5  = $f2->Frame()->pack(-side => 'left');
  $f6  = $f2->Frame()->pack(-side => 'right');

  $w0  = $f4->Label(-text => 'Dependent Variable')->pack(-side => 'top');
  $w1  = $f4->BrowseEntry(-variable=> \$dv)->pack(-side => 'bottom');

  $w4  = $f5->Label(-text => 'Independent Variables')->pack(-side => 'top');
  $w5  = $f5->Scrolled('Listbox',-scrollbars=>'oe',-activestyle => 'dotbox', -selectmode => 'multiple')->pack(-side => 'bottom');
  $w5->bind('<<ListboxSelect>>', sub { @ind1 = $w5->curselection(); });

  $w9  = $f6->Label(-text => 'Weighting')->pack(-side=>'top');
  $w2  = $f6->Checkbutton(-text => 'Apply Weighting',-variable => \$do_weight)->pack(-side => 'top');
  $w3  = $f6->BrowseEntry(-variable=> \$wv)->pack(-side => 'bottom');
  
  $w6  = $f3->Button(-text => 'Regress!',-command => $tk_Regress_s0)->pack(-side => 'left');
  $w7   = $f3->Button(-text => 'cancel',-command => sub { $tbor->withdraw(); })->pack(-side => 'right');
  $w8  = $f3->Checkbutton(-text => 'Use Robust',-variable => \$do_robust)->pack(-side=>'bottom');
 } else {
  $tbor->deiconify();
  $tbor->raise();
 }

 $w1->delete(0,'end');
 $w3->delete(0,'end');
 $w5->delete(0,'end');
 foreach my $v (sort @vars) {
  $w1->insert('end',$v);
  $w3->insert('end',$v);
  $w5->insert('end',$v);
 }
 $w5->activate(0);  
}

sub tk_Exclusionrestrict {
  tk_handleCommand("exclusionrestrict");   
}

sub tk_About {

 my $mw = $tktree->{main};

 my $ta;
 my $f1;
 my $f2;
 my $f1l;
 my $f1r;
 my $l1;
 my $b1;
 my $ro2;
 my $ro1;
 my $about_text;

 if (!Exists($ta)) {
  $ta = $mw->Toplevel();
  $about_text = "\nAT-ATS (c)2006-2008\nDavid Jacobowitz\ndavid\@kellobowitz.net\nhttp://www.kellobowitz.net/david";
  $ta->title("AT-ATS About");
  $f1 = $ta->Frame()->pack(-side => 'top',-expand=>'1',-fill=>'both');
  $f2 = $ta->Frame()->pack(-side => 'bottom',-expand=>'1',-fill=>'both'); 
  $f1l = $f1->Frame()->pack(-side=> 'left',-expand=>'1',-fill=>'both');
  $f1r = $f1->Frame()->pack(-side=> 'right',-expand=>'1',-fill=>'both');
  $l1 = $f1l->Label(-text => $about_text,-justify=>'l',-font=>'about_large')->pack(-side => 'top');
  # $ro2 = $f1r->ROText(-width=>'20',-height=>'4');
  my $version_string = "\n\n" . 
                       "Version: " . $gb->{version_major} . ":" . 
                                     $gb->{version_minor} . ":" . 
				     $gb->{build_number} . "\n" .
                                     $gb->{build_date} . "\n" . 
				     $gb->{installdir};
  $ro2 = $f1r->Label(-text => $version_string, -justify=>'r')->pack(-side=>'top');
  # $ro2->insert('end',$version_string);

  $b1 = $f2->Button(-text => 'Close', -command => sub { $ta->withdraw(); })->pack(-side => 'bottom');
  if (defined($commands->{help}{help})) {
   $ro1 = $f2->Scrolled('ROText',-scrollbars=>'e',-width=>'50',-height=>'10',-wrap=>'word')->pack(-side=>'top',-expand=>'1',-fill=>'both');
   foreach my $l (@{$commands->{help}{help}{about}}) {
    $ro1->insert('end',$l . "\n");
   }
  }
  $ta->deiconify();
  $ta->raise();
 }
}


sub tk_Help {
 my $mw = $tktree->{main};
 my $tbor;
 my $lb1; 
 my $ro1;

 # this functions populates the help window with the help topic
 # that was selected from the listbox on the left
 sub tk_help_s0 { 
  my $lb1   = shift;
  my $hdata = shift;
  my $ro1   = shift;
  my $tbor  = shift;
  my $topic = $lb1->get($lb1->curselection());
  $ro1->delete("1.0",'end');
  $ro1->insert('end',"\nHelp: $topic\n",'title');
  $tbor->configure(-title=>"AT-ATS Help: $topic");
  foreach my $l (@{$hdata->{$topic}}) {
   if ($l =~ /^\s*\.[\s\w]*/) {
    $ro1->insert('end',$l . "\n",'cmd');
   } elsif ($l =~ /help:\w*/) {
    my @pieces = split(/\s/,$l);
    foreach my $piece (@pieces) {
     if ($piece =~ s/(.*)help:(\w*)(.*)//) {
      $ro1->insert('end',$1);
      my $topic = $2;
      $ro1->insert('end',$topic,'help_help');
      $ro1->insert('end',$piece . $3. " ");
     } else {
      $ro1->insert('end',$piece . " ");
     }
    }
   } else {
    $ro1->insert('end',$l . "\n");
   }
  }
 };

 # this function repopulates the help window with the help topic
 # that was just clicked on from within the help window itself
 # (the topic comes from a chunk of text previously tagged as
 # "help_help"
 sub tk_help_s1 { 
  my $tg = shift;
  my $hdata = shift;
  my $tbor  = shift;
  my $ro1   = shift;
  my $xy    = shift;

  my $ind   = $ro1->index($xy);
  my @range = $ro1->tagPrevrange('help_help',$ind);
  my $topic = $ro1->get(@range);
  $topic =~ s/\s*//g;
  $ro1->delete("1.0",'end');
  $ro1->insert('end',"\nHelp: $topic\n",'title');
  $tbor->configure(-title=>"AT-ATS Help: $topic");
  foreach my $l (@{$hdata->{$topic}}) {
   if ($l =~ /^\s*\.[\s\w]*/) {
    $ro1->insert('end',$l . "\n",'cmd');
   } elsif ($l =~ /help:\w*/) {
    my @pieces = split(/\s/,$l);
    foreach my $piece (@pieces) {
     if ($piece =~ s/^help:(\w*)//) {
      my $topic = $1;
      $ro1->insert('end',$topic,'help_help');
      $ro1->insert('end',$piece. " ");
     } else {
      $ro1->insert('end',$piece . " ");
     }
    }
    $ro1->insert('end',"\n");
   } else {
    $ro1->insert('end',$l . "\n");
   }
  }
 };


 if (!Exists($tbor)) {
  $tbor = $mw->Toplevel();
  my $help_text = "Click on a topic\nbelow to see some\nbasic help at right";
  $tbor->title("AT-ATS Help");
  my $f1   = $tbor->Frame()->pack(-side => 'top',-expand=>'1',-fill=>'both');
  my $f1l  = $f1->Frame()->pack(-side => 'left',-expand=>'0',-fill=>'y');
  my $f1r  = $f1->Frame()->pack(-side => 'right',-expand=>'1',-fill=>'both');
  my $l1   = $f1l->Label(-text => $help_text)->pack(-side => 'top');
     $lb1  = $f1l->Scrolled('Listbox',-scrollbars=>'e',-selectmode=>'single')->pack(-side=>'top',-expand=>'1',-fill=>'y');
  my $b1 = $f1l->Button(-text => 'Close', -command => sub { $tbor->withdraw(); })->pack(-side => 'bottom');
  $ro1  = $f1r->Scrolled('ROText',-scrollbars=>'e',-width=>60,-height=>'20',-wrap=>'word')->pack(-side=>'bottom',-expand=>'1',-fill=>'both');
  $ro1->tag('configure','title', -font=>'help_title');
  $ro1->tag('configure','cmd'  , -font=>'help_cmd',-foreground=>'#00c0c0');
  $ro1->tag('configure','help_help', -font=>'C_bold',-foreground=>'cyan');
  $ro1->tagBind('help_help','<1>', [\&tk_help_s1, $commands->{help}{help}, $tbor, $ro1, Ev('@')]);
  $ro1->tagBind('help_help','<Any-Enter>' => sub {shift->tagConfigure('help_help',-relief=>'raised',-borderwidth=>'2') });
  $ro1->tagBind('help_help','<Any-Leave>' => sub {shift->tagConfigure('help_help',-relief=>'flat')});
  $ro1->insert('end',"Choose a topic at left.",'title');
  if (defined($commands->{help}{help})) {
   $lb1->delete(0,'end');
   foreach my $topic (sort keys %{$commands->{help}{help}}) {
    $lb1->insert('end',$topic);
    $lb1->bind('<<ListboxSelect>>', [ \&tk_help_s0, $commands->{help}{help}, $ro1, $tbor ]);
   }
  } else {
   $lb1->insert('end',"<missing>");
  }
 } else {
  $tbor->deiconify();
  $tbor->raise();
 }


}

sub tk_Anova_oneway {
 my $mw = $tktree->{main};

 my $F1;
 my $F1t;
 my $F1b;
 my $F2l;
 my $F2r;
 my $LBL1;
 my $LBL2;
 my $LB1;
 my $LB2;
 my $F3l;
 my $F3r;
 my $BT1;
 my $CK1;
 my $tbor;

 my $var1; my $var2;
 my @vars = sort keys %{$dm->{data}};
 my @nvars = ();
 foreach my $v (@vars) {
  if ($dm->{info}{$v}{type} eq "numeric") {
   push(@nvars,$v);
  }
 }

 my $tk_Anova_oneway_s0 = sub {
  my $cmd = "anova " . $var1 . " " .  $var2;
  $tbor->withdraw();
  tk_handleCommand($cmd);
 };

 if (!Exists($tbor)) {
  $tbor= $mw->Toplevel();
  $tbor->title("Oneway ANOVA");
  $F1 = $tbor->Frame()->pack(-side => 'top');
  $F1t = $F1->Frame()->pack(-side => 'top');
  $F1b = $F1->Frame()->pack(-side => 'bottom');
  $F2l = $F1t->Frame()->pack(-side => 'left');
  $F2r = $F1t->Frame()->pack(-side => 'right');
  $LBL1 = $F2l->Label(-text => 'Continuous Variable Select')->pack(-side => 'top');
  $LB1  = $F2l->BrowseEntry(-variable => \$var1)->pack(-side => 'bottom');
  $LBL2 = $F2r->Label(-text => 'Categorical Variable Select')->pack(-side => 'top');
  $LB2  = $F2r->BrowseEntry(-variable => \$var2)->pack(-side => 'bottom');
  $F3l  = $F1b->Frame()->pack(-side => 'left');
  $F3r  = $F1b->Frame()->pack(-side => 'right');
  $BT1  = $F3l->Button(-text => 'tabulate',-command => $tk_Anova_oneway_s0)->pack(-side => 'bottom');
 } else {
  $tbor->deiconify();
  $tbor->raise();
 }

 $LB1->delete(0,'end');
 $LB2->delete(0,'end');
 foreach my $v (sort @nvars) {
  $LB1->insert('end',$v);
 }
 foreach my $v (sort @vars) {
  $LB2->insert('end',$v);
 }
}


sub tk_Ttest_twovar {
 my $mw = $tktree->{main};
 my $var1, my $var2;
 my $tbor;
 my $be1;
 my $be2;
 my $uneq = 0;
 my $unpaired = 0;
 my $use_if  = 0;
 my $en1;
 my $tk_Ttest_twovar_s0 = sub {
  my $cmd = "ttest " . $var1 . " == " . $var2;
  if ($use_if) { $cmd .= " if " . $en1->get() . " "; };
  if ($unpaired || $uneq) { $cmd .= ", "; }
  if ($unpaired) { $cmd .= "unpaired "; }
  if ($uneq) { $cmd .= "unequal "; }
  $tbor->withdraw();
  tk_handleCommand($cmd);
 };


 if (!Exists($tbor)) {
  $tbor = $mw->Toplevel();
  $tbor->title("Two Variable T-Test");
  my $f1   = $tbor->Frame()->pack(-side=>'top');
  my $f2   = $tbor->Frame()->pack(-side=>'bottom');
  my $f3   = $tbor->Frame()->pack(-side=>'bottom');
  my $f1l  = $tbor->Frame()->pack(-side=>'left');
  my $f1r  = $tbor->Frame()->pack(-side=>'right');
  my $lbl1 = $f1l->Label(-text=> 'Choose First Variable')->pack(-side=>'top');
  my $lbl2 = $f1r->Label(-text=> 'Choose Second Variable')->pack(-side=>'top');
  $be1  = $f1l->BrowseEntry(-variable => \$var1)->pack(-side=>'bottom');
  $be2  = $f1r->BrowseEntry(-variable => \$var2)->pack(-side=>'bottom');
  my $ck1  = $f2->Checkbutton(-text => 'Unequal variances',-variable => \$uneq)->pack(-side=>'left');
  my $ck2  = $f2->Checkbutton(-text => 'Unpaired data',-variable => \$unpaired)->pack(-side=>'left');
  my $bt1  = $f2->Button(-text => 'Go!',-command=> $tk_Ttest_twovar_s0)->pack(-side=>'right');
  my $ck3  = $f3->Checkbutton(-text => 'Filter (if)',-variable => \$use_if)->pack(-side=>'left');
  $en1  = $f3->Entry(-state=>'disabled',-width=>40)->pack(-side=>'left');
  $ck3->bind('<ButtonPress>', sub { if ($use_if) { $en1->configure(-state=>'normal'); } else { $en1->configure(-state=>'disabled'); } });
  $tbor->deiconify;
  $tbor->raise;
 }
 $be1->delete(0,'end');
 $be2->delete(0,'end');
 my @vars = sort keys %{$dm->{data}};
 foreach my $v (sort @vars) {
  $be1->insert('end',$v);
  $be2->insert('end',$v);
 }
}

sub tk_Ttest_onevar {
 my $mw = $tktree->{main};
 my $var1, my $var2;
 my $tbor;
 my $be1;
 my $be2;
 my $uneq = 0;
 my $unpaired = 0;
 my $use_if  = 0;
 my $en1;
 my $tk_Ttest_onevar_s0 = sub {
  my $cmd = "ttest " . $var1;
  if ($use_if) { $cmd .= " if " . $en1->get() . " "; };
  $cmd .= ", by(" . $var2 . ")";
  # if ($unpaired || $uneq) { $cmd .= ", "; }
  if ($unpaired) { $cmd .= "unpaired "; }
  if ($uneq) { $cmd .= "unequal "; }
  $tbor->withdraw();
  tk_handleCommand($cmd);
 };


 if (!Exists($tbor)) {
  $tbor = $mw->Toplevel();
  $tbor->title("Two Variable T-Test");
  my $f1   = $tbor->Frame()->pack(-side=>'top');
  my $f2   = $tbor->Frame()->pack(-side=>'bottom');
  my $f3   = $tbor->Frame()->pack(-side=>'bottom');
  my $f1l  = $tbor->Frame()->pack(-side=>'left');
  my $f1r  = $tbor->Frame()->pack(-side=>'right');
  my $lbl1 = $f1l->Label(-text=> 'Choose Variable')->pack(-side=>'top');
  my $lbl2 = $f1r->Label(-text=> 'Choose bin/\"by\" Variable')->pack(-side=>'top');
  $be1  = $f1l->BrowseEntry(-variable => \$var1)->pack(-side=>'bottom');
  $be2  = $f1r->BrowseEntry(-variable => \$var2)->pack(-side=>'bottom');
  my $ck1  = $f2->Checkbutton(-text => 'Unequal variances',-variable => \$uneq)->pack(-side=>'left');
  my $ck2  = $f2->Checkbutton(-text => 'Unpaired data',-variable => \$unpaired)->pack(-side=>'left');
  my $bt1  = $f2->Button(-text => 'Go!',-command=> $tk_Ttest_onevar_s0)->pack(-side=>'right');
  my $ck3  = $f3->Checkbutton(-text => 'Filter (if)',-variable => \$use_if)->pack(-side=>'left');
  $en1  = $f3->Entry(-state=>'disabled',-width=>40)->pack(-side=>'left');
  $ck3->bind('<ButtonPress>', sub { if ($use_if) { $en1->configure(-state=>'normal'); } else { $en1->configure(-state=>'disabled'); } });
  $tbor->deiconify;
  $tbor->raise;
 }
 $be1->delete(0,'end');
 $be2->delete(0,'end');
 my @vars = sort keys %{$dm->{data}};
 foreach my $v (sort @vars) {
  $be1->insert('end',$v);
  $be2->insert('end',$v);
 }
}


sub tk_ScatterPlot {
 my $mw = $tktree->{main};
 my $tbor;
 my $Xvar;
 my $Yvar;
 my $Zvar;
 my ($be1, $be2, $be3, $be4);
 my $fname = "";
 my $fmt   = "emf";
 my $do_if = 0;
 my ($en1, $en2);
 my $tk_ScatterPlot_s0 = sub {
  my $ok = (defined($Xvar) && length($Xvar) &&
            defined($Yvar) && length($Yvar));
  my $z3d = $ok && defined($Zvar) && length($Zvar);


  $fname = $en1->get();
  my $cmd = "scatter ";
  if ($ok) {
   $cmd .= $Xvar . " " . $Yvar . " ";
  }

  if ($z3d) {
   $cmd .= $Zvar . " ";
  }

  if (defined($fname) && length($fname)) {
   $cmd .= "[output=$fname] ";
  }

  $cmd .= "[format=$fmt] ";

  if ($do_if) {
   $cmd .= "if " . $en2->get() . " ";
  }

  if ($ok) {
   tk_handleCommand($cmd); 
  }
  $tbor->withdraw();
 };

 my $tk_ScatterPlot_s1 = sub {
  $tbor->withdraw();
 };

 if (!Exists($tbor)) {
  $tbor = $mw->Toplevel();
  $tbor->title("Scatter Plot");
  my $f1    = $tbor->Frame()->pack(-side=>'top');
  my $f1l   = $f1->Frame()->pack(-side=>'left');
  my $f1r   = $f1->Frame()->pack(-side=>'right');
  my $lbl1  = $f1l->Label(-text=>'Choose Variables')->pack(-side=>'top');
  my $f1l1  = $f1l->Frame()->pack(-side=>'top');
  my $f1l2  = $f1l->Frame()->pack(-side=>'top');
  my $f1l3  = $f1l->Frame()->pack(-side=>'top');
  my $f1l1l = $f1l1->Frame()->pack(-side=>'left');
  my $f1l1r = $f1l1->Frame()->pack(-side=>'left');
  my $f1l2l = $f1l2->Frame()->pack(-side=>'left');
  my $f1l2r = $f1l2->Frame()->pack(-side=>'left');
  my $f1l3l = $f1l3->Frame()->pack(-side=>'left');
  my $f1l3r = $f1l3->Frame()->pack(-side=>'left');
  my $lbl2  = $f1l1l->Label(-text=>'X')->pack(-side=>'top');
     $be1   = $f1l1r->BrowseEntry(-variable=> \$Xvar)->pack(-side=>'top');
  my $lbl3  = $f1l2l->Label(-text=>'Y')->pack(-side=>'top');
     $be2   = $f1l2r->BrowseEntry(-variable=> \$Yvar)->pack(-side=>'top');
  my $lbl4  = $f1l3l->Label(-text=>'Z')->pack(-side=>'top');
     $be3   = $f1l3r->BrowseEntry(-variable=> \$Zvar)->pack(-side=>'top');
  my $f1r1  = $f1r->Frame()->pack(-side=>'top');
  my $f1r2  = $f1r->Frame()->pack(-side=>'top');
  my $f1r3  = $f1r->Frame()->pack(-side=>'top');
  my $f1r4  = $f1r->Frame()->pack(-side=>'bottom');
  my $lbl5  = $f1r1->Label(-text=>'File Name')->pack(-side=>'left');
     $en1   = $f1r1->Entry(-state=>'normal',-width=>40)->pack(-side=>'right');
  my $lbl6  = $f1r2->Label(-text=>'Format')->pack(-side=>'left');
     $be4   = $f1r2->BrowseEntry(-variable=>\$fmt)->pack(-side=>'right');
#  my $ck1   = $f1r3->Checkbutton(-text => 'If condition',-variable => \$do_if)->pack(-side=>'left');
#     $en2   = $f1r3->Entry(-state=>'disabled',-width=>40)-pack(-side=>'right');
#  $ck1->bind('<ButtonPress>', sub { if ($do_if) {
#                                     $en2->configure(-state=>'normal');
#				    } else {
#                                     $en2->configure(-state=>'disabled');
#				    }
#	                          } );
  my $bt1   = $f1r4->Button(-text => 'Go!',-command=> $tk_ScatterPlot_s0)->pack(-side=>'left');
  my $bt2   = $f1r4->Button(-text=>'Cancel',-command => $tk_ScatterPlot_s1)->pack(-side=>'right');

 }
 $be1->delete(0,'end');
 $be2->delete(0,'end');
 $be3->delete(0,'end');
 my @vars = sort keys %{$dm->{data}};
 foreach my $v (sort @vars) {
  $be1->insert('end',$v);
  $be2->insert('end',$v);
  $be3->insert('end',$v);
 }
 $be4->delete(0,'end');
 foreach my $f (@{$gb->{graphics_formats}}) {
  $be4->insert('end',$f);
 }
}





sub tk_Histogram {
 my $mw = $tktree->{main};
 my $tbor;
 my $discrete = 0;
 my $normal   = 0;
 my $type     = "emf";
 my $fname;
 my $var1     = "";
 my $use_if;
 my $bincount = 10;
 my $en1;
 my $en2;
 my $be1;
 my $be2;
 my $be3;
 my $tk_Histogram_s0 = sub {
  my $cmd = "histogram " . $var1 . " ";
  $fname = $en2->get();
  $cmd .= "[output=$fname] ";
  $cmd .= "[format=$type] ";
  if (!$discrete) {
   $cmd .= "[bincount=$bincount] ";
  }
  if ($use_if) {
   $cmd .= "if (" . $en1->get() . ") ";
  }
  if ($discrete || $normal) {
   $cmd .= ", ";
  }
  if ($discrete) { $cmd .= "discrete "; };
  if ($normal)   { $cmd .= "normal"; };
  $tbor->withdraw();
  tk_handleCommand($cmd);
 };

 my $tk_Histogram_s1 = sub {
  $tbor->withdraw();
 };

 if (!Exists($tbor)) {
  $tbor = $mw->Toplevel();
  $tbor->title("Histogram Plot");
  my $f1   = $tbor->Frame()->pack(-side=>'top');
  my $f2   = $tbor->Frame()->pack(-side=>'top');
  my $f3   = $tbor->Frame()->pack(-side=>'top');
  my $f4   = $tbor->Frame()->pack(-side=>'top');
  my $f5   = $tbor->Frame()->pack(-side=>'bottom');

  my $lbl1 = $f1->Label(-text=> 'Choose Variable')->pack(-side=>'left');
  $be1  = $f1->BrowseEntry(-variable => \$var1)->pack(-side=>'left');

  my $lbl2 = $f2->Label(-text=>'File Name')->pack(-side=>'left');
  $en2     = $f2->Entry(-state=>'normal',-width=>40)->pack(-side=>'left');
  my $lbl3 = $f2->Label(-text=>'Type')->pack(-side=>'left');
  $be2     = $f2->BrowseEntry(-variable => \$type)->pack(-side=>'right');

  my $ck2  = $f3->Checkbutton(-text => 'Add Normal Curve',-variable => \$normal)->pack(-side=>'left');
  my $ck1  = $f3->Checkbutton(-text => 'Discrete Bins',-variable => \$discrete)->pack(-side=>'left');
  $ck1->bind('<ButtonPress>', sub { if ($normal) { $normal = 0; } 
                                    if ($discrete) {
                                     $be3->configure(-state=>'disabled');
				    } else {
                                     $be3->configure(-state=>'normal');
				    }
	                          } );
  $ck2->bind('<ButtonPress>', sub { if ($discrete) { 
			             $discrete = 0; 
				     $be3->configure(-state=>'normal') 
			            } 
			          } );
  $be3     = $f3->BrowseEntry(-variable => \$bincount)->pack(-side=>'left');
  my $ck3  = $f4->Checkbutton(-text => 'Filter (if)',-variable => \$use_if)->pack(-side=>'left');
  $ck3->bind('<ButtonPress>', sub { if ($use_if) { $en1->configure(-state=>'normal'); } else { $en1->configure(-state=>'disabled'); } });
  $en1  = $f4->Entry(-state=>'disabled',-width=>40)->pack(-side=>'left');

  my $bt1  = $f5->Button(-text => 'Go!',-command=> $tk_Histogram_s0)->pack(-side=>'left');
  my $bt2  = $f5->Button(-text=>'Cancel',-command => $tk_Histogram_s1)->pack(-side=>'right');

  $en2->delete(0,'end');
  $en2->insert(0,"dummy");
  $be2->delete(0,'end');
  foreach my $f (@{$gb->{graphics_formats}}) {
   $be2->insert('end',$f);
  }

  $be3->delete(0,'end');
  $be3->insert('end',"2");
  $be3->insert('end',"3");
  $be3->insert('end',"4");
  $be3->insert('end',"5");
  $be3->insert('end',"10");
  $be3->insert('end',"15");
  $be3->insert('end',"20");
  $be3->insert('end',"25");
  $be3->insert('end',"50");
  $be3->insert('end',"100");
  $tbor->deiconify;
  $tbor->raise;
 }
 $be1->delete(0,'end');
 my @vars = sort keys %{$dm->{data}};
 foreach my $v (sort @vars) {
  $be1->insert('end',$v);
 }
}

1;


