#!/usr/bin/perl -w

#   Copyright (C) 2006 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

package logging;
use Exporter 'import';
@EXPORT_OK = qw(new);

use Data::Dumper;


use misc2 qw(inAry inDict);
use misc qw(step min max);
use Scalar::Util qw(looks_like_number);
use eqn_string qw(create_evaluable_eqn);

#nothing implemented right now!
sub my_wrap {
 return shift;
}

sub new {
 my $proto           = shift;
 my $self            = {};
 my $class           = ref($proto) || $proto;

 $self->{stderr}     = undef;
 $self->{stdout}     = *STDOUT;
 $self->{logout}     = undef;
 $self->{logopen}    = 0;
 $self->{logfname}   = 0;
 $self->{stderropen} = 0;
 $self->{gui}        = 0;
 $self->{quiet}      = 0;
 bless ($self,$class);
 return $self;
}

sub commands {
 my $self = shift;
 my @cmds = ("log");
 return @cmds;
}

sub set_windowout {
 my $self   = shift;
 my $tk_win = shift;
 $self->{tkreswin} = $tk_win;
 $self->{gui}      = 1;
}

sub set_stderr {
 my $self = shift;
 my $arg  = shift;
 if (defined($arg)) {
  $self->{stderr}     = $arg;
  $self->{stderropen} = 1;
 } else {
  $self->{stderropen} = 1;
 }
}

sub supress {
 my $self = shift;
 my $arg  = shift;

 if (defined($arg) && looks_like_number($arg) && $arg) {
  $self->{quiet} = 1; 
 } else {
  $self->{quiet} = 0; 
 }
}



sub log {
 my $self = shift;
 my $a    = shift;
 my $r    = { status => "ok" };

 # print Dumper($a);
 if (!$a->{firstvar}{present}) {
  if ($a->{have_using}) {
   my $fn = $a->{using};
   $fn =~ s/^"//;
   $fn =~ s/"$//;
   if ($self->{logopen}) {
    my $ofn = $self->{logfname};
    $self->disp("> Log is already open. ($ofn)\n");
    $self->disp("> Closing.\n");
    my $lo = $self->{logout};
    close $lo;
    $self->{logopen} = 0;
   }
   my $ok = open (LF, ">" . $fn);
   if ($ok) {
    $self->{logfname} = $fn;
    $self->{logout}  = *LF;
    $self->{logopen} = 1; 
    $self->disp("Log opened on $fn\n");
    $self->logonly("----------------------------------------\n");
    $self->logonly("       log : $fn\n");
    $self->logonly("      type : text\n");
    my $ds = localtime(time);
    $self->logonly(" opened on : $ds\n\n\n");
    return $r;
   } else {
    return { status => "fail; could not open new log file ($fn)" };
   }
  } {
   return { status => "fail; no filename supplied" };
  }
 } elsif ($a->{firstvar}{present} && ($a->{firstvar}{name} eq "close")) {
  if ($self->{logopen}) {
   my $lo = $self->{logout};
   my $fn = $self->{logfname};
   $self->disp("> Closing logfile $fn\n");
   close $lo;
   $self->{logopen} = 0;
  }
 } else {
  return { status => "fail; garbled command" };
 }
 return $r;
}



sub debug {
 my $self = shift;
 my $msg  = shift;

 if ($self->{stderropen}) {
  my $se = $self->{stderr};
  print $se $msg;
 }
}

# this will only output to the GUI if in use, otherwise
# it does nothing
sub gdisp {
 my $self = shift;
 my $msg  = shift;
 if (!$self->{quiet}) {
  if ($self->{gui}) {
   my $tw = $self->{tkreswin};
   my $tag = "";
   if ($msg =~ /^\> fail/) {
    $tag = "error";
   } elsif ($msg =~ /^\>?\s*warn;/) {
    $tag = "warning";
   } elsif ($msg =~ /^\./) {
    $tag = "command";
   } elsif ($msg =~ /^\>/) {
    $tag = "message";
   }
   if ($self->{gui}) {
    $tw->insert('end',$msg,$tag);
    $tw->see('end');
   }
  }
  if ($self->{logopen}) {
   my $lo = $self->{logout};
   print $lo $msg;
  }
 }
}

# this will only output to stdout if stdout is in use,
# otherwise it won't do anything
sub cdisp {
 my $self = shift;
 my $msg  = shift;
 if (!$self->{quiet} && !$self->{gui}) {
  my $so = $self->{stdout};
  print $so my_wrap($msg);
 }
}

### normal display routine; write out to a log, or the gui
### or the standard out if using that
sub disp {
 my $self = shift;
 my $msg  = shift;

 if (!$self->{quiet}) {
  if ($self->{gui}) {
   my $tw = $self->{tkreswin};
   my $tag = "";
   if ($msg =~ /^\> fail/) {
    $tag = "error";
   } elsif ($msg =~ /^\>?\s*warn;/) {
    $tag = "warning";
   } elsif ($msg =~ /^\.[\s\w*].*/) {
    $tag = "command";
   } elsif ($msg =~ /^\>/) {
    $tag = "message";
   }
   if ($self->{gui}) {
    $tw->insert('end',$msg,$tag);
    $tw->see('end');
    $tw->update();
   }
  } else {
   my $so = $self->{stdout};
   print $so my_wrap($msg);
  }
  if ($self->{logopen}) {
   my $lo = $self->{logout};
   print $lo my_wrap($msg);
  }
 }
}

### normal display, but with a specific tag specified by the caller
### for the TK/gui. Text mode ignores the tag.
sub tdisp {
 my $self = shift;
 my $tag  = shift;
 my $msg  = shift;
 if (!$self->{quiet}) {
  if ($self->{gui}) {
   my $tw = $self->{tkreswin};
   $tw->insert('end',$msg,$tag);
   $tw->see('end');
   $tw->update();
  } else {
   my $so = $self->{stdout};
   print $so my_wrap($msg);
  }
  if ($self->{logopen}) {
   my $lo = $self->{logout};
   print $lo my_wrap($msg);
  }
 }
}

### this will display to the stdout or gui, depending on which
### is in use, but will bypass the log
sub nolog {
 my $self = shift;
 my $msg  = shift;
 if (!$self->{quiet}) {
  if ($self->{gui}) {
   my $tw = $self->{tkreswin};
   $tw->insert('end',$msg);
   $tw->see('end');
  } else {
   my $so = $self->{stdout};
   print $so my_wrap($msg);
  }
 }
}

### this only goes to the log 
sub logonly {
 my $self = shift;
 my $msg  = shift;
 if (!$self->{quiet} && $self->{logopen}) {
  my $lo = $self->{logout};
  print $lo my_wrap($msg);
 }
}

sub done {
 my $oo = $self->{oo};
 if ($self->{logopen}) {
  $oo->disp("> Closing logfile.\n");
  my $lo = $self->{logout};
  close $lo;
 }
 if ($self->{stderropen}) {
  $oo->disp("> Closing debugfile.\n");
  my $se = $self->{stderr};
  close $se;
 }
}

1;

