#!/usr/bin/perl -w

#   Copyright (C) 2006-2008 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

package misc;

use warnings 'all';
use strict   'refs';

use Exporter qw(import);
use POSIX qw(floor);
use misc2 qw(func_fail);
use Data::Dumper;
use dataqual qw(gen_defquald_a);

use Scalar::Util qw(looks_like_number);
use Math::Random qw(random_set_seed_from_phrase random_normal);

@EXPORT_OK = qw(banner one_var_stats covariance 
                fixedFloat fixedFloat_p numCount step min max
		xtile randnorm genTabArray deep_copy);


##  xtinDict is just like the misc::inDict, except that 
## because xtile is called in an eval in an unknown context, 
## calls to inDict don't work.
sub xtinDict {
 my $lf = shift;
 my $d  = shift;
 my $in = 0;
 if (!defined($d)) { return 0; };
 if (!defined($lf)) { return $in; };
 foreach $de (keys %{$d}) {
  if ($de eq $lf) {
   $in = 1;
  }
 }
 return $in;
}

my $randnorm_init = 0;
sub randnorm {
 my $mu     = shift;
 my $sigma  = shift;
 
 if (!$randnorm_init) {
  $randnorm_init = 1;
  random_set_seed_from_phrase(scalar time);
 }
 if (!defined($mu))    { $mu    = 0; };
 if (!defined($sigma)) { $sigma = 1; };
 return random_normal(1,$mu,$sigma);
}




sub xtile {
 my $self = shift;
 my $row  = shift;
 my $bc   = shift;
 my $var  = shift;

 if (defined($var) && xtinDict($var,$self->{dm}{data})) {
  # print "XTIL____ ROW $row BC $bc VAR $var\n";
  if ($row == 0) {
   $self->{xtile}{min} = undef;
   $self->{xtile}{max} = undef;
   for (my $i=0;$i<@{$self->{dm}{data}{$var}};$i++) {
    my $v = $self->{dm}{data}{$var}[$i];
    if (looks_like_number($v)) {
     if (!looks_like_number($self->{xtile}{max}) ||
	 ($v > $self->{xtile}{max})) {
      $self->{xtile}{max} = $v;	  
     }
     if (!looks_like_number($self->{xtile}{min}) ||
         ($v < $self->{xtile}{min})) {
      $self->{xtile}{min} = $v;
     }
    }
   }
  }
  # print "MIN $self->{xtile}{min} MAX $self->{xtile}{max}\n";
  my $range   = $self->{xtile}{max} - $self->{xtile}{min};
  my $ptile;
  if (!looks_like_number($self->{dm}{data}{$var}[$row])) {
   return "__undef";
  }
  if ($range) {
   $ptile   = 100*($self->{dm}{data}{$var}[$row] - $self->{xtile}{min}) / ($self->{xtile}{max}-$self->{xtile}{min});
  } else {
   $ptile   = 0;
  }
  my $pperbin = 100/$bc;
  my $bin     = $ptile / $pperbin;
  if ($ptile == 100) {
   # no 100th percentile; just 99.999999
   $bin = $bc-1;
  }
  # print "bin $bin ptile $ptile val $self->{dm}{data}{$var}[$row] min $self->{xtile}{min} max $self->{xtile}{max} range $range\n";
  $bin = int($bin);
  return $bin;
 } else {
  $self->{oo}->disp("Error in xtile arguments.\n");
  return -1;
 }
}


sub numCount {
 my $ar = shift;
 my @a  = @{$ar};
 my $c  =0;
 foreach my $e (@a) {
  if (looks_like_number($e)) { $c++; };
 }
 return $c;
}






sub banner {
 my $gb = shift;
 my $ver_maj = $gb->{version_major};
 my $ver_min = $gb->{version_minor};
 my $bld_dat = $gb->{build_date};
 my $bld_num = $gb->{build_number};
 my $idir    = $gb->{installdir};

 my $os = << "__banner";
                                                
           x                    x            
           x                    x            
  xxxx    xxxx         xxxx    xxxx    xxxxx    Copyright (C) 2006-2008
      x    x               x    x     x     x   David Jacobowitz
  xxxxx    x     xxxx  xxxxx    x      xxx   
 x    x    x          x    x    x         xx    Send questions to:
 x    x    x  x       x    x    x  x  x     x     david\@kellobowitz.net
  xxxx x    xx         xxxx x    xx    xxxxx      
                                                    
    "Association does not imply causation."
                                                
 Version      : $ver_maj:$ver_min:$bld_num
 Build Date   : $bld_dat
 Install Dir  : $idir;
 Perl Version : $]
 This software is distributed for free, under the terms of the
 GNU Public License and comes with ABSOLUTELY NO WARRANTY. For 
 details, type "help license".

__banner

 return $os;
}


sub percentile {
 my $dref    = shift;
 my $ca      = shift;
 
 my @d = sort { $a <=> $b } @{$dref};
 my $count = scalar @d;
 my $res = [];

 foreach my $centile (@{$ca}) { 
  my $sample = ($count) * ($centile/100);
  my $lsample = floor($sample);
  my $hsample = floor($sample+1);
  my $f       = $sample - $lsample;
  my $value = (($d[$lsample] * (1-$f)) + ($d[$hsample] * ($f)));
  push (@{$res}, $value); 
 }
 return $res;
}

sub covariance {
 my $xref = shift;
 my $yref = shift;
 my @pre_x    = @{$xref};
 
 my $status = "ok";
 my @pre_y    = @{$yref};
 if (!@pre_x || !@pre_y) {
  return { status => "fail; empty arrays" };
 }
 if ($#pre_x != $#pre_y) {
  return { status => "fail; arrays unequal lengths" };
 }

 my @x = (); 
 my @y = ();
 for (my $i=0;$i<@pre_x;$i++) {
  if (looks_like_number($pre_x[$i]) && looks_like_number($pre_y[$i])) {
   push(@x,$pre_x[$i]);  
   push(@y,$pre_y[$i]);  
  }
 }

 my $xtot = 0;
 my $ytot = 0;
 my $rows = $#x + 1;
 for (my $i=0;$i<$rows;$i++) {
  $xtot += $x[$i]; 
  $ytot += $y[$i]; 
#  print "X $x[$i] Y $y[$i]\n"
 }

 my $xmean = $xtot / $rows;
 my $ymean = $ytot / $rows;

 my $sxy = 0;
 my $sxx = 0;
 my $syy = 0;
 for (my $i=0;$i<$rows;$i++) {
  $sxy += ($x[$i] - $xmean)*($y[$i] - $ymean);
  $sxx += ($x[$i] - $xmean)**2;
  $syy += ($y[$i] - $ymean)**2;
 }
 my $covariance = $sxy/($rows-1);
 my $x_variance = $sxx/($rows-1);
 my $y_variance = $syy/($rows-1);
 my $x_sd       = sqrt($x_variance);
 my $y_sd       = sqrt($y_variance);
 my $corr       = 0;
 if (!$x_sd || !$y_sd) {
  $corr       = "__undef";
  $status     = "fail; covariance analysis fails; no variation in one variable";
 } else {
  $corr       = $covariance/($x_sd*$y_sd);
 }
 if (0) {
  print "covary_summary:\n";
  print " cov  $covariance\n";
  print " corr $corr\n";
  print " x_sd $x_sd\n";
  print " y_sd $y_sd\n";
  print "\n";
 };
 return { covariance  => $covariance,
	  x_variance  => $x_variance,
	  y_variance  => $y_variance,
	  x_std_dev   => $x_sd,
	  y_std_dev   => $y_sd,
	  x_mean      => $xmean,
	  y_mean      => $ymean,
	  correlation => $corr,
          status      => $status };

}






sub one_var_stats {
 my $dref  = shift;
 my @data  = @{$dref};
 my $r     = {};
 $r->{status} = "ok";

 my $count = 0;
 my $min   = $data[0];
 my $max   = $data[0];
 my $sum   = 0;
 my @newary = ();
 foreach my $datum (@data) {
  if (looks_like_number($datum)) {
   if ($datum < $min) { $min = $datum; };
   if ($datum > $max) { $max = $datum; };
   $sum += $datum;
   push @newary,$datum;
   $count++;
  }
 }

 if ($count == 0) {
  $r->{status} = "fail; no elements in array";
  return $r;
 }
 if ($count == 1) {
  $r->{status} = "warn; cannot gen statitics on single observation";
  $r->{mean}     = $newary[0];
  $r->{max}      = $newary[0];
  $r->{min}      = $newary[0];
  $r->{count}    = 1;
  $r->{variance} = "na";
  $r->{std_dev}  = "na";
  $r->{kurtosis} = "na";
  $r->{skewness} = "na";
  $r->{median}   = $newary[0];
  return $r;
 }


 my $mean = $sum / $count;
 $r->{mean}  = $mean;
 $r->{count} = $count;
 $r->{min}   = $min;
 $r->{max}   = $max;

 my $ss = 0;
 my $sc = 0;
 my $sq = 0;
 @newary = sort @newary;
 for (my $i=0;$i<$count;$i++) {
  my $datum = $newary[$i];
  my $mdiff = $datum - $mean;
  my $square = $mdiff**2;
  my $cube   = $mdiff**3;
  my $quad   = $mdiff**4;
# I don't know why, but multiplying is slower than the powers.
#  my $square = $mdiff  * $mdiff;
#  my $cube   = $square * $mdiff;
#  my $quad   = $cube   * $mdiff;
  $ss += $square;
  $sc += $cube;
  $sq += $quad;
 }
 my $variance = $ss / ($count - 1);
 $r->{variance} = $variance;
 my $std_dev    = sqrt($variance);

 $r->{std_dev}  = $std_dev;
 my $skewness   = "na";
 if ($ss != 0) {
  $skewness = ((sqrt($count)*$sc) / ($ss**1.5)) - 0;
 }
 my $kurtosis   = "na";
 if ($ss != 0) {
  $kurtosis = (($count * $sq) / ($ss**2)) - 0;
 }
 $r->{skewness} = $skewness;
 $r->{kurtosis} = $kurtosis;

 $r->{std_err}  = $std_dev / sqrt($count);

 my $percentiles = percentile(\@newary, [0.1,1,5,10,25,50,75,90,95,99,99.9]);

 $r->{p001}     = $percentiles->[0];
 $r->{p01}      = $percentiles->[1];
 $r->{p05}      = $percentiles->[2];
 $r->{p10}      = $percentiles->[3];
 $r->{p25}      = $percentiles->[4];
 $r->{median}   = $percentiles->[5];
 $r->{p75}      = $percentiles->[6];
 $r->{p90}      = $percentiles->[7];
 $r->{p95}      = $percentiles->[8];
 $r->{p99}      = $percentiles->[9];
 $r->{p999}     = $percentiles->[10];
 return $r;
}

sub fixedFloat {
 my $num    = shift;
 my $width  = 10;
 my $spaces = "                              ";

 my $t = "";

 if (!looks_like_number($num)) {
 } else {
  if ((abs($num) < 0.000001) ||
      (abs($num) > 1000000)) {
   $t = sprintf("% 1.2e",$num);
  } else {
   $t = sprintf("% 4.7f",$num);
  }
 }

 if (length($t) > $width) {
  $t = substr($t,0,$width);
 } elsif (length($t) < $width) {
  $t .= substr($spaces,0,$width-length($t));
 }
 return $t;
}

sub fixedFloat_p {
 my $num    = shift;
 my $width  = 10;
 my $spaces = "                              ";
 my $t = "";
 if (!looks_like_number($num)) {
 } else {
  $num = abs($num);
  if (($num < 0.000001) ||
      ($num > 1000000)) {
   $t = sprintf("%1.2e",$num);
  } else {
   $t = sprintf("%4.7f",$num);
  }
 }
 if (length($t) > $width) {
  $t = substr($t,0,$width);
 } elsif (length($t) < $width) {
  $t .= substr($spaces,0,$width-length($t));
 }
 return $t;
}



sub step {
 my $a0 = shift;
 if    ($a0 > 0) { return 1; }
 elsif ($a0 < 0) { return 0; }
 else            { return 0.5; };
};

sub max {
 my $a0 = shift;
 my $a1 = shift;
 if ($a0 < $a1) { return $a1; } else { return $a0; };
};

sub min {
 my $a0 = shift;
 my $a1 = shift;
 if ($a0 < $a1) { return $a0; } else { return $a1; };
};


# for a list of observations of a variable, generate a hash
# of all the values that appear, where the value of each element
# is the number of times that value appears; 
sub genTabArray {
 $dm  = shift;
 $var = shift;
 my $lh = {};

 my $qa = gen_defquald_a($dm, $var);
 for (my $row=0;$row<@{$qa};$row++) {
  if (defined($qa->[$row])) {
   my $val = $qa->[$row];
   if (defined($lh->{$val})) { $lh->{$val} += 1; }
   else { $lh->{$val} = 1; }
  }
 }
 return $lh;
}


sub deep_copy {
 my $this = shift;
 my $rr = ref $this;
 if (not ref $this) {
  $this;
 } elsif (ref $this eq "ARRAY") {
  [map deep_copy($_), @$this];
 } elsif (ref $this eq "HASH") {
   +{map { $_ => deep_copy($this->{$_}) } keys %$this};
 } elsif (ref $this eq "Math::Pari") {
  $this;
 } elsif (ref $this eq "PDL") {
##  pdl $this;
 } else { # we only deep copy hashes and arrays; not objects we don't
	  # understand
 }
}


1;

